<?php

namespace HCM\FPA\Calculos\Rescisao;

use HCM\FPA\Calculos\Geral\Calculo;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\FPA\Entity\FpaTipodireitoEntity;
use HCM\FPA\Entity\FpaformrescisaoEntity;
use HCM\Util\MsgCalculo;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaPtregraEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\FPA\Entity\FpaTpdemissaoEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpeAltestabvincEntity;
use HCM\Geral\Entity\GpevinculohEntity;
use HCM\FPA\Entity\FpaParamfolhahEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaFormcalculoEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use Teknisa\Model\Gestaopessoas\Repositories\FpaValorreferenciam;
use HCM\FPA\Calculos\Geral\FGTS;
use HCM\FPA\Calculos\Geral\INSS;
use HCM\FPA\Calculos\Geral\IRRF;

class CalculoRescisao extends Calculo {

    public function calculaRescisao($vinculo, $tp_calc_mov, $contador){
        
        if($this->dataSet['DSORDEM'] == "LISTA_LOTE"){
            if(!is_array($this->dataSet['DSINICIO'])){
                $nrlotevincs = explode(',',$this->dataSet['DSINICIO']);
            } else {
                $nrlotevincs = $this->dataSet['DSINICIO'];
            }
            
            foreach($nrlotevincs as $nrlotevinc){
                $vinculoEncontrado = $this->entityManager->getRepository(Repositories::FPA_ITEMLOTEVINC)->findOneBy(array('nrorg' => $this->_nrorg, 'nrlotevinc' =>  $nrlotevinc, 'nrvinculom' => $vinculo->getNrvinculom()));

                if(!empty($vinculoEncontrado)){
                    $this->_nrlotevinc = $nrlotevinc;
                    continue;
                }
            }
        }
        
        $alte_estab = new GpeAltestabvincEntity($this->entityManager, $this->hcmParameters);
        $estabilidade = $alte_estab->verificaEstabilidadeCalculo($this->_nrorg, $vinculo->getNrvinculom(), $vinculo->getDtrescisaovinc());

        if ($estabilidade) {
            MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui estabilidade.');
            //return;
        }
        
        $calcula = $this->_inicializaVariaveis($vinculo, $tp_calc_mov);
        
        if(!$calcula){
            return;
        }
        
        if ($tp_calc_mov->retornaItensCalculo($vinculo) === false) {
            return;
        }
        
        try {
            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
            $this->_itens_calculo_principal =  $obj_tp_calc_mov->getInstancia();
            $this->_itens_calculo_principal->setNrTipoCalculo($this->_nr_tipo_calculo);
        } catch(Exception $e) {
            $message = $e->getMessage();
            $message .= ' - CALCULO.PHP';
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }
        
        if($this->tpProcessamento == FPA::PROC_RECALCPATRONAL){
            $this->_calculaGrupoIncidencias();
            $tp_calc_mov->persisteItensCalculo(null, $this->_nrlotevinc);
            return;
        }
        
        $liquido = $this->_controleLiquido();
        if ($liquido == true) {
            MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.', MsgCalculo::ERRO);
            return;
        }
        
        $this->_controleLiquidoParcial();
        $this->_inicializaVariaveisRegra();
        $this->_calculo_regra->geraVariaveis();
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_INICIO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        //Gera debito nas rescisao.
        $_mesCompetenciaAnterior = DateUtil::subtraiIntervalo($this->_mes_competencia, 1, DateUtil::MES);   
        $calc_folhaAnterior = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
            $this->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAnterior, $_mesCompetenciaAnterior
        )[0];
        
        if($calc_folhaAnterior){
            $itemCalcCompetenciaAnterior = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAnterior->getNrcalculofolha());
    
            $itemCalcCompetenciaAnterior->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAnterior->getNrcalculofolha());
    
            $evento_difSalarioLiquido = $this->_eventom->retornaEvento(760);
            $val = $itemCalcCompetenciaAnterior->retornaValorPorEvento($evento_difSalarioLiquido, null);
            $evento_desconto = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_COMPLEMENTO_SAL_DB);
            $valDeb = $itemCalcCompetenciaAnterior->retornaValorPorEvento($evento_desconto, null);
            
            if($evento_difSalarioLiquido && $val > 0 && !$valDeb){
                $eventoAuxDeb = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_COMPLEMENTO_SAL_DB, $this->_eventom, $this->_nrorg
                );
                $this->_itens_calculo_principal->geraEvento($eventoAuxDeb, $this->_mes_competencia, $val, 0, 1);
            }
        }
        
        $this->_verificaTrabalhoEfetivo($vinculo);
        
        MemoriaCalculo::getInstance()->criaCabecalho($this->entityManager, $this->hcmParameters, $this->_vinculo,$this->_nr_tipo_calculo,$this->_sit_func_resumo,$this->_salario,$this->_dias_uteis,$this->_dias_rsr,
            $this->_mes_competencia, $this->_dt_pagamento, $this->_nrOcorrenciaCalculo,$this->_obj_tipo_calculo->getNmtipocalculo(),$this->_gozo_ferias);
        
        $this->_geraReferenciaSitFunc();
        
        if ($this->_nr_tipo_calculo !== FpatipocalculoEntity::CALC_FOLHA &&
            $vinculo->getNrtpdemissao()==FpaTpdemissaoEntity::SEM_JUSTA_CAUSA_EMPREGADOR &&
            $vinculo->getDtrescisaovinc()!= $vinculo->getDtavisoprevio()){
            $this->_calculaDsrSuplementar($this->_itens_calculo_principal, $vinculo);
        }
        
        $this->_calculaRemuneracaoBase($vinculo, $this->_itens_calculo_principal, $this->_nr_tipo_calculo);
        
        
        if($this->_idDifSal === 'S'){
            $this->_verificaDifSalarial($this->_mes_competencia, $vinculo);
        }
        
        if($this->_idPagaAdicional === 'S'){
            $this->_pagaAdicionalFerias();
        }

        $this->_demonstraAdiantamento($tp_calc_mov);
        
        if (($this->_nr_tipo_calculo === FpatipocalculoEntity::CALC_RESCISAO)
                || ($this->_nr_tipo_calculo === FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR)
            ) {
                if($this->_ultPagamento === FpaFormcalculoEntity::IDULTPGTOMES_S) {
                    $this->_demonstraIRRFCaixa();
                }
                $this->_demonstra13Salario($tp_calc_mov);
            }
        
        $this->_demonstraFerias($tp_calc_mov);
        
        if($this->_idDifFerias === 'S') {
            $this->_verificaDifFerias($tp_calc_mov, $vinculo);
        }
        
        $this->_calculaLicencaRemun();

        $this->_calculaValeTransporte();

        if((int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL]>0 ||
            (int)$this->_sit_func_resumo[GpesitufuncmEntity::SITUACAO_FUNCIONAL_MATERNIDADE]>0)
        {
            $this->_calculaContribuicaoSindical($vinculo);
        }

        $this->_analisaSalMaternidade($vinculo, $tp_calc_mov);
        
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_ANTES_CALC_ESPECIFICO, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );    
        
        switch($this->_nr_tipo_calculo){
            case FpatipocalculoEntity::CALC_RESCISAO: $this->_executaRescisao($vinculo, $tp_calc_mov); break;
            case FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR: $this->_executaRescisaocomplementar($vinculo, $tp_calc_mov); break;
        }
        
        $this->_calculaSalarioFamilia($vinculo);

        $this->_calculaGrupoIncidencias();
        
        $this->_geraPensao();
            
        $this->_calculaTotais();
        
        if (($this->_recalculo === true) || ($this->_nrOcorrenciaCalculo > 1)) {

            if($this->_nr_tipo_calculo===FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR){

                $comp_ocorr_resc = DateUtil::setDay($vinculo->getDtrescisaovinc(), 1);
            
                if($this->_mes_competencia->format('d/m/Y') === $comp_ocorr_resc->format('d/m/Y')){
                    if($this->_nrOcorrenciaCalculo==1){
                        MsgCalculo::getInstance()->adicionaMensagem('Para fazer Rescisao Complementar dentro do mês de competência da Rescisão o Nr de Ocorrencia deve ser maior que '.$this->_nrOcorrenciaCalculo);
                    } else {
                        $calculo_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha($this->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1,
                        $this->_nrOcorrenciaCalculo-1, $comp_ocorr_resc,$comp_ocorr_resc);
                    }
                }elseif($this->_mes_competencia > $comp_ocorr_resc){
                    $calculo_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)
                        ->retornaCalculoFolha(
                            $this->_nrorg,
                            FpatipomovimentEntity::TP_MOV_MENSAL,
                            1,
                            1,
                            $comp_ocorr_resc,
                            $comp_ocorr_resc);
                }
                $listaNrCalculo = array();
                foreach ($calculo_folha as $calculo){
                    $listaNrCalculo[] = $calculo->getNrcalculofolha();
                }
                $this->_itens_origem->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $listaNrCalculo[0]);

                $eventoAuxCred = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_COMPLEMENTO_SAL_CR , $this->_eventom, $this->_nrorg
                );
                $recuperavalor=$this->_itens_origem->retornaValorPorEvento($eventoAuxCred, $comp_ocorr_resc);

                $eventoAuxDeb = $this->_assoc_eventos->retornaEventoAssociado(
                    FpatipoassoceveEntity::ASSOC_COMPLEMENTO_SAL_DB , $this->_eventom, $this->_nrorg
                );
                if ($recuperavalor>0) {
                    MemoriaCalculo::getInstance()->escreveEventos($this->_itens_calculo_principal->getNrcalculofolha(), $eventoAuxDeb,null, $this->_mes_competencia, $recuperavalor,0,null,null,null,100);
                    $this->_itens_calculo_principal->geraEvento($eventoAuxDeb, $this->_mes_competencia, $recuperavalor, 0, 1);
                }

                if($this->_id_fazDifComp== FpaformrescisaoEntity::ID_FAZDIFERENCACOMP_SIM){

                    $eventoAux = $this->_assoc_eventos->retornaEventoAssociado(
                        FpatipoassoceveEntity::ASSOC_SALARIO_1 , $this->_eventom, $this->_nrorg
                    );
                    $recuperareferencia=$this->_itens_origem->retornaReferenciaPorEvento($eventoAux, $comp_ocorr_resc);
                    MemoriaCalculo::getInstance()->escreveEventos($this->_itens_calculo_principal->getNrcalculofolha(), $eventoAux,null, $this->_mes_competencia,1, $recuperareferencia,null,null,null,100);
                    $this->_itens_calculo_principal->geraEvento($eventoAux, $this->_mes_competencia,0, $recuperareferencia,1);

                    $this->_calculaSalario($this->_itens_calculo_principal);

                    $eventos_transf = $this->_itens_origem->preparaRecalculoProvEventual( $this->_itens_origem, $this->_eventom);
                    for ($i=0; $i < count($eventos_transf); $i++) {
                        $evento_aux = $this->_eventom->retornaEvento($eventos_transf[$i]['NREVENTO']);
                        if ($eventos_transf[$i]['VALOR']>0) {
                            MemoriaCalculo::getInstance()->escreveEventos($this->_itens_calculo_principal->getNrcalculofolha(), $eventoAux,null, $eventos_transf[$i]['DTOCORRENCIA'], $eventos_transf[$i]['VALOR'], $eventos_transf[$i]['REFERENCIA'],null,null,null,100);
                            $this->_itens_calculo_principal->geraEvento($evento_aux, $eventos_transf[$i]['DTOCORRENCIA'],  $eventos_transf[$i]['VALOR'], $eventos_transf[$i]['REFERENCIA'], 1);
                        }
                    }

                    $this->_calculaFaltas($this->_itens_calculo_principal);

                    $this->_calculaAdicionais($this->_itens_calculo_principal);

                    $this->_calculo_regra->executaPontoRegra(
                        $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_ADICIONAIS, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                    );
                    
                    $this->_calculaReflexo($this->_itens_calculo_principal, FpatipoincideEntity::INC_REFLEXO_ADIC_NOTURNO, FpatipoassoceveEntity::ASSOC_REFLEXO_ADIC_NOTURNO);

                    $this->_calculaExtra($this->_itens_calculo_principal);

                    $this->_calculo_regra->executaPontoRegra(
                        $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_HORAS_EXTRAS, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                    );
                    
                    $this->_calculaReflexo($this->_itens_calculo_principal, FpatipoincideEntity::INC_REFLEXO_HORA_EXTRA, FpatipoassoceveEntity::ASSOC_REFLEXO_REP_REMUNERADO);

                }
                $this->_itens_calculo_principal->calculaDiferencaRecalculo($this->_itens_origem, $this->_eventom, $this->_id_fazDifComp);


                if ($vinculo->getNrvinculoempreg() == GpevinculohEntity::VINCULO_EMPREGATICIO_MENOR_APRENDIZ) {
                    $percentual = GpevinculohEntity::PERCENTUAL_FGTS_MENOR_APRENDIZ;
                } else{
                    $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS);
                }
                FGTS::calcula(
                    FpatipoincideEntity::INC_FGTS_RESC, FpatipoassoceveEntity::ASSOC_FGTS_RESC_DEPOSITADO,
                    FpatipoassoceveEntity::ASSOC_INCID_FGTS_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                    $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                );
                
                if (in_array(FpaTipodireitoEntity::FGTS_PAGO, $this->_listaDireitos) === true){
                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_13RESC, FpatipoassoceveEntity::ASSOC_FGTS_13SAL_RESC,
                        FpatipoassoceveEntity::ASSOC_INCID_FGTS_13_RESC, $this->_mes_competencia,
                        $this->_itens_calculo_principal, $this->_assoc_eventos, $this->_eventom, $percentual,
                        $this->_nrorg
                    );
                }
                
                if (in_array(FpaTipodireitoEntity::FGTS_DEPOSITADO, $this->_listaDireitos) === true){
                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_13RESC, FpatipoassoceveEntity::ASSOC_FGTS_13_RESCISAO,
                        FpatipoassoceveEntity::ASSOC_INCID_FGTS_13_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );
                }

                if (in_array(FpaTipodireitoEntity::MULTA_FGTS_PAGO, $this->_listaDireitos) === true){
                    $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS_RESC_MULTA);
                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_IND_RESCISAO_ENC, FpatipoassoceveEntity::ASSOC_FGTS_IND_RESCISAO,
                        FpatipoassoceveEntity::ASSOC_FGTS_RESC_DEPOSITADO, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );

                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_IND_BANCO_ENC, FpatipoassoceveEntity::ASSOC_FGTS_IND_BANCO,
                        FpatipoassoceveEntity::ASSOC_FGTS_TOTAL_BANCO, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );
                }

                if (in_array(FpaTipodireitoEntity::MULTA_FGTS_DEPOSITADO, $this->_listaDireitos) === true){
                    $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::FGTS_RESC_MULTA);
                    
                    if($vinculo->getNrtpdemissao() == FpaTpdemissaoEntity::ACORDO_ENTRE_PARTES){
                        $percentual = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, FpaValorreferenciamEntity::FGTS_RESC_MULTA_ACORDO);
                    }
                    
                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_IND_RESCISAO_ENC, FpatipoassoceveEntity::ASSOC_FGTS_IND_RESCISAO_ENC,
                        FpatipoassoceveEntity::ASSOC_FGTS_RESC_DEPOSITADO, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );

                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_IND_BANCO_ENC, FpatipoassoceveEntity::ASSOC_FGTS_IND_BANCO_ENC,
                        FpatipoassoceveEntity::ASSOC_FGTS_TOTAL_BANCO, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );

                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_13RESC, FpatipoassoceveEntity::ASSOC_FGTS_IND_13_RESCISAO_ENC,
                        FpatipoassoceveEntity::ASSOC_INCID_FGTS_13_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual * 8/100, $this->_nrorg
                    );
                }

                if (in_array(FpaTipodireitoEntity::MULTA_CONTRIBUICAO_SOCIAL, $this->_listaDireitos) === true){
                    $percentual = $this->_valores_referencia->getValorReferencia(FpaValorreferenciamEntity::CONTRIB_SOC_RESC);
                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_CONTR_SOC_RESC_ENC, FpatipoassoceveEntity::ASSOC_FGTS_CONTR_SOC_RESC_ENC,
                        FpatipoassoceveEntity::ASSOC_FGTS_RESC_DEPOSITADO, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );

                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_CONTR_SOC_BANCO_ENC, FpatipoassoceveEntity::ASSOC_FGTS_CONTR_SOC_BANCO_ENC,
                        FpatipoassoceveEntity::ASSOC_FGTS_TOTAL_BANCO, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual, $this->_nrorg
                    );

                    FGTS::calcula(
                        FpatipoincideEntity::INC_FGTS_13RESC, FpatipoassoceveEntity::ASSOC_FGTS_CONTR_SOC_13_RESC_ENC,
                        FpatipoassoceveEntity::ASSOC_INCID_FGTS_13_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $percentual * 8/100, $this->_nrorg
                    );
                }

                $this->_calculo_regra->executaPontoRegra(
                    $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_FGTS, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                );
                
                //SOMA BASE DAS OCORRENCIAS E FAZ DIFERENCA DOS INSS.
                if($this->_nrOcorrenciaCalculo > 1){
                    
                    $vrEventoAntigo = null;
                    $vrEventoNovo = null;
                    
                    INSS::calcula(
                        FpatipoincideEntity::INC_INSS_RESC, FpatipoassoceveEntity::ASSOC_INSS_RESCISAO,
                        FpatipoassoceveEntity::ASSOC_INCID_INSS_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $this->_nrorg, null,null,null,0,false, $this->_itens_origem
                    );
                    
                    $evento = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_INSS_RESCISAO);
                    $indice = $this->_itens_origem->procuraEvento($evento);
                    $itemAntigo = $this->_itens_origem->getItem($indice);
                    if($itemAntigo){
                        $vrEventoAntigo = $itemAntigo->getVreventofolha();
                    }
                    
                    $indice = $this->_itens_calculo_principal->procuraEvento($evento);
                    $itemNovo = $this->_itens_calculo_principal->getItem($indice);
                    if($itemNovo){
                        $vrEventoNovo = $itemNovo->getVreventofolha();
                    }
                    
                    if($vrEventoNovo && $vrEventoAntigo){
                        
                        INSS::calcula(
                            FpatipoincideEntity::INC_INSS_RESC, FpatipoassoceveEntity::ASSOC_INSS_RESCISAO,
                            FpatipoassoceveEntity::ASSOC_INCID_INSS_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                            $this->_assoc_eventos, $this->_eventom, $this->_nrorg, null,null,null,0,false
                        );
                        
                        $inss = $vrEventoNovo - $vrEventoAntigo < 0 ? 0 : $vrEventoNovo - $vrEventoAntigo;
                        
                        $this->_itens_calculo_principal->geraEvento($evento, null, $inss, 0, 1);
                    }
                }else{
                    INSS::calcula(
                        FpatipoincideEntity::INC_INSS_RESC, FpatipoassoceveEntity::ASSOC_INSS_RESCISAO,
                        FpatipoassoceveEntity::ASSOC_INCID_INSS_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                        $this->_assoc_eventos, $this->_eventom, $this->_nrorg, null,null,null,0,false
                    );
                }
                
                INSS::calcula(
                    FpatipoincideEntity::INC_INSS_13RESC, FpatipoassoceveEntity::ASSOC_INSS_13SAL_RESC,
                    FpatipoassoceveEntity::ASSOC_INCID_INSS_13_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                    $this->_assoc_eventos, $this->_eventom, $this->_nrorg, null,null,null,0,false
                );

                $this->_calculo_regra->executaPontoRegra(
                    $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_INSS, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                );

                IRRF::calcula(
                    FpatipoincideEntity::INC_IRRF_RESC, FpatipoassoceveEntity::ASSOC_IRRF_RESCISAO,
                    FpatipoassoceveEntity::ASSOC_INCID_IRRF_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                    $this->_assoc_eventos, $this->_eventom, $this->_valores_referencia, $vinculo, $this->_idIRAcumulaRecMinimo
                );

                IRRF::calcula(
                    FpatipoincideEntity::INC_IRRF_13RESC, FpatipoassoceveEntity::ASSOC_IRRF_13SAL_RESC,
                    FpatipoassoceveEntity::ASSOC_INCID_IRRF_13_RESC, $this->_mes_competencia, $this->_itens_calculo_principal,
                    $this->_assoc_eventos, $this->_eventom, $this->_valores_referencia, $vinculo, $this->_idIRAcumulaRecMinimo
                );
                $this->_calculo_regra->executaPontoRegra(
                    $this->_idDebug, FpaPtregraEntity::PTREGRA_APOS_IRRF, $this->_nr_tipo_calculo, $this->_itens_calculo_principal
                );

                $this->_calculaGrupoIncidencias();

                $this->_calculaTotais();

            }
        }
        $this->_calculo_regra->executaPontoRegra(
            $this->_idDebug, FpaPtregraEntity::PTREGRA_FINAL, $this->_nr_tipo_calculo,$this->_itens_calculo_principal
        );
        
        MemoriaCalculo::getInstance()->setMovimentos();
        MemoriaCalculo::getInstance()->setIncidencias();
        
        $arquivoMemoria = MemoriaCalculo::getInstance()->getNmArquivo();
        
        $lista = null;
        
        if($arquivoMemoria){
            $lista['ARQUIVOMEMORIA'] = FPA::getTempDirectory().'/'.$arquivoMemoria;
        }
        
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            return $lista;
        }
        if ($this->_simulacao === false) {
            $tp_calc_mov->persisteItensCalculo(null, $this->_nrlotevinc);
            return $lista;
        } else {
            $lista['SIMULACAO'] = $tp_calc_mov->retornaListaItensCalculo(
                $this->_mes_competencia, $this->_eventom, $vinculo
            );
            //Vinculo
            if(count($lista) > 0){
                $lista['SIMULACAO'][0][] = array(
                    'MESOCORRENCIA' => $vinculo->getNrvinculom(),
                    'TIPOEVENTO' => $vinculo->getNmvinculom(),
                    'NRFILHO' => $vinculo->getNrvinculom(),
                    'NREVENTOM' => 'Matricula: '.$vinculo->getCdmatricula(),
                    'NMEVENTOH' => 'Salario: ' .number_format($this->_salario, 2, ',','.'),
                    'UNIDREFEEVEN' => 'Admissão: '.substr($vinculo->getDtadmissaovinc(), 0, 10),
                    'VRREFFOLHA' => null,
                    'IDEVENDEMON' => null,
                    'VREVENTOFOLHA' => null,
                    'NRORDEIMPRES' => null,
                    'ORDEMTIPOEVENTO' => null,
                    'NRVINCULOM' => null,
                    'NRCALCULOFOLHAVINC' => null
                );    
            }
            
            return $lista;
        }
    }
}