<?php

namespace HCM\FPA\Calculos\FolhaComplementar;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Entity\TipohierarqorgEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\GpeParamoperEntity;
use HCM\Geral\Entity\GpealtesalarioEntity;
use HCM\FPA\Services\CalculoFolhaService;
use HCM\FPA\Entity\FpatpcalcmovEntity;
use HCM\FPA\Entity\FpaincideventoEntity;
use HCM\FPA\Entity\FpatipoincideEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FparesumoctblEntity;
use HCM\FPA\Entity\FpaProcessoEntity;
use HCM\FPA\Entity\FpaTpvincprocessoEntity;
use HCM\FPA\Entity\FpaDireitotpdemisEntity;
use HCM\FPA\Entity\FpatipocalculoEntity;
use HCM\FPA\Calculos\Geral\Inicializa;
use HCM\FPA\Calculos\Folha\CalculoFolha;
use HCM\FPA\Calculos\Geral\Dependencia;
use HCM\FPA\Calculos\Geral\MemoriaCalculo;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use Teknisa\Model\Gestaopessoas\Repositories\FpaCalculoconsumer;

class PercorreFolhaComplementar{

    private $_parametros;
    private $_nrOrgPadrao;
    private $_nrOrgTrabalho;
    private $_nrEstruturaTrab;
    private $_idDebug;
    private $_ambienteSalarioOperador;
    private $_nrTipoModalidadeCalculo;
    private $_nrOcorrenciaCalculo;
    private $_mesCompetencia;
    private $_nrTipoCalculo;
    private $_simulacao;
    private $_retrocesso;
    private $_parcial;
    private $_dtPagamento;
    private $_ultPagamento;
    private $_dt_inicio;
    private $_dt_fim;
    private $_dsInicio;
    private $_dsFim;
    private $_id_retrocesso;
    private $_id_fazDifComp;
    private $_dsOrdem;
    private $_tipo_calculo;
    private $_calculo_folha;
    private $_tp_calc_mov;
    private $_vinculom;
    private $_tpVinculoProcesso;
    private $_tpRescisaoDireito;
    private $_pessoa;
    private $_dependencia;
    private $_formulario;
    private $_msg_idstatus;
    private $_msg_idabort;
    private $_msg_mensagens;
    private $_cadGozoFerias;

    public function __construct($entityManager, $hcmParameters, $dataSet){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->preparaComplementar($dataSet);
        $this->_parametros = $hcmParameters;
        $this->_mesCompetencia = $hcmParameters->getDtCompetencia();
        $this->_nrOrgTrabalho = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_nrEstruturaTrab = FPA::getEstruturaTrabalho();
        $this->_retrocesso = false;
        $this->_parcial = false;
        $this->_cadGozoFerias = false;
        $this->tpProcessamento = strtoupper($dataSet['TP_PROCESSAMENTO']);
        
        switch ($this->tpProcessamento){
            case FPA::PROC_SIMULACAO :
                $this->_simulacao = true; break;
            case FPA::PROC_CALCULO :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR :
                $this->_simulacao = false; break;
            case FPA::PROC_APOLO_GRAVAR_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_RETROCESSO :
                $this->_retrocesso = true; break;
            case FPA::PROC_CALCULO_PARCIAL :
                $this->_simulacao = false;
                $this->_parcial = true; break;
            case FPA::PROC_CADGOZOFERIAS :
                $this->_cadGozoFerias = true; break; 
            default:
                throw new \Exception('Tipo de processo '.$this->tpProcessamento.' invalido');
        }

        Inicializa::Container($entityManager, $hcmParameters, $this->_dtPagamento);
        
        $this->_vinculom = TkDI::getContainer()->vinculom;
        $this->_tipo_calculo = TkDI::getContainer()->tipo_calculo;
        $this->_calculo_folha = TkDI::getContainer()->calculo_folha;
        $this->_pessoa = TkDI::getContainer()->pessoa;
        $this->_tp_calc_mov = TkDI::getContainer()->tp_calc_mov;
        $this->_dependencia = new Dependencia($entityManager, $hcmParameters);
        $this->_tpVinculoProcesso = new FpaTpvincprocessoEntity($entityManager, $hcmParameters);
        $this->_tpRescisaoDireito = new FpaDireitotpdemisEntity($entityManager, $hcmParameters);
        $this->objTipoCalculo = new FpatipocalculoEntity($entityManager, $hcmParameters);
        $this->_alteracaoSalario = new GpealtesalarioEntity($entityManager, $hcmParameters);
        $this->_eventom = TkDI::getContainer()->eventom;
        $this->_assoc_eventos = TkDI::getContainer()->assoc_eventos;

    }
    
    public function preparaComplementar($dataSet){
        $this->_nrTipoCalculo = $dataSet['TP_CALCULO'];
        $this->_buscaDadosFormularioCalculo($dataSet);
        $this->dataSet = $dataSet;
    }
    
    public function percorre(){
        
        $objTpCalc = $this->objTipoCalculo->retornaTipoCalculo($this->_nrTipoCalculo);
        $mesInicio = $this->_dtInicio->format('m');
        $mesFim = $this->_dtFim->format('m');
        $ocorrencia = 1;
        $this->_nrTipoModalidadeCalculo = 13; //Cenario III
        $tp_calc_movs = [];
        
        for($i = $mesInicio; $i < $mesFim; $i++){
            $tp_calc_movs[$i] = new FpatpcalcmovEntity($this->entityManager, $this->hcmParameters);
            $tp_calc_movs[$i]->retornaTipoCalculoMovimento($this->_nrTipoCalculo); 
            
            $competencia = DateUtil::getDataDeString("01/".$i."/".$this->_dtInicio->format("Y"));
            
            $tp_calc_movs[$i]->setCompetencia($competencia);
            
            $tp_calc_movs[$i]->atribuiNrCalculoFolha(
                $this->_nrOrgTrabalho, $this->_calculo_folha, $competencia, $this->_nrTipoModalidadeCalculo,
                $ocorrencia
            );
        }
        $this->_tpVinculoProcesso->carregaTipoVinculoPorProcesso(
            $this->_nrOrgTrabalho, $this->_nrOrgPadrao, $this->_mesCompetencia
        );
        
        $vinculos = $this->_vinculom->retornaVinculosPorTipoCalculo($this->_nrTipoCalculo, $this->_nrOrgTrabalho, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim,
            $this->_dt_inicio, $this->_dt_fim, $this->_dsOrdem, $this->_nrEstruturaTrab);
        
        $qtd_vinculo = count($vinculos);
        
        if ($qtd_vinculo > 0) {
            
            $resultado = array();
            $contador = 0;
            $idstatus = MsgCalculo::ID_PENDENTE;
            $idabort = false;
            
            $resultadoCalcular = null;
            foreach($tp_calc_movs as $tp_calc_mov){
                $dtCompetencia = $tp_calc_mov->getCompetencia();
                $this->hcmParameters->set("dtmescompetenc", substr($dtCompetencia, 0, 10));
                $this->_eventom->setMesCompetencia(substr($dtCompetencia, 0, 10));
                
                Inicializa::carregaVariaveis();
                
    			$calculo = new CalculoFolhaComplementar($this->entityManager, $this->hcmParameters, $this->dataSet, 1, $this->_simulacao, $this->_parcial, $dtCompetencia);
                
                $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
                $nrcalcprincipal = $obj_tp_calc_mov->getNrCalcFolha();
                $_mesCompetenciaAtual = DateUtil::getDataDeString($this->_mesCompetencia, DateUtil::FORMATO_BRASILEIRO, true);    
                foreach ($vinculos as $key => $vinculo) {
                    
                    $carregou = $this->_alteracaoSalario->carregaAlteracaoSalario($_mesCompetenciaAtual,  $vinculo->getNrvinculom(), 1);
                    $dtAlteracao = $this->_alteracaoSalario->getDtalterasalario();
                    $dataBase = $this->_alteracaoSalario->getDataBaseDissidio();
                    
                    if(!$dataBase){
                        continue;
                    }
                    
                    if($dtAlteracao->format('d/m/Y') != $_mesCompetenciaAtual->format('d/m/Y')
                    or $dtCompetencia < $dataBase){
                        continue;
                    }
                    
                    //Verifica liquido
                    $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                        $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
                    )[0];
                    $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
                    $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
                    $liquido = $itemCalcCompetenciaAtual->verificaLiquido(
                        $this->_assoc_eventos, $this->_eventom, $this->_nrorg
                    );
                    
                    MsgCalculo::getInstance()->setDados($nrcalcprincipal, $vinculo->getNrvinculom(), $this->_nrOrgTrabalho, 'ERROSCALCULO');
                    
                    if ($liquido == true) {
                        MsgCalculo::getInstance()->adicionaMensagem('Vinculo possui liquido.');
                        $this->finalizaMensagens();
                        continue;
                    }
                    
                    if($contador >= 10 && $this->_simulacao==true){
                        MsgCalculo::getInstance()->adicionaMensagem("Simulado e demonstrado apenas os 10 primeiros vínculos da lista. Para calcular mais vínculos será necessário gravar.");
                        break;
                    }
                    $listaProcessos = $this->_tpVinculoProcesso->getItem($vinculo->getNrtipovinculom());
                        
                    if (is_null($listaProcessos)) {
                        MsgCalculo::getInstance()->adicionaMensagem('Tipo de vínculo não possui processos de cálculo');
                        if (!$this->finalizaMensagens()) {
                            continue;
                        }
                        continue;
                    }
                        
                    if ($this->_retrocesso === false) {
                        //ITENS SALARIO ANTIGO
                        $itensCalculoOrig = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters);
                        $calc_folha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                            $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $dtCompetencia, $dtCompetencia
                        )[0];
                        
                        $itensCalculoOrig->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folha->getNrcalculofolha());
                        $itemsOrig = $itensCalculoOrig->getItems();
                        
                        if(count($itemsOrig) <= 0){
                            MsgCalculo::getInstance()->adicionaMensagem('Vinculo não possui liquido na competencia '.$dtCompetencia);
                            $this->finalizaMensagens();
                            continue;
                        }
                        
                        //ITENS SALARIO NOVO
                        $tp_calc_mov->retornaItensCalculo($vinculo);
                        $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
                        $_itens_calculo_principal = $obj_tp_calc_mov->getInstancia();
                        $_itens_calculo_principal->setNrTipoCalculo(1);
                        $itemsPrincipal = $_itens_calculo_principal->getItems();
                        //LIMPA SEMPRE ITENS NOVOS
                        if(count($itemsPrincipal) > 0){
                            $_itens_calculo_principal->removeAllItens();
                        }
                        
                        $incideventoEntity = new FpaincideventoEntity($this->entityManager, $this->hcmParameters);
                        $incideEventosNaoRecalcula = $incideventoEntity->getEventos(
                            $this->_mesCompetencia, FpatipoincideEntity::NÃO_RECALCULA_DISSIDIO
                        );
                        
                        $incideEventosRecalculaPerc = $incideventoEntity->getEventos(
                            $this->_mesCompetencia, FpatipoincideEntity::RECALCULA_POR_DISSIDIO
                        );
                        
                        $eventosInseridos = [];
                        //Replica a folha com salario antigo para o novo calculo.
                        foreach($itemsOrig as $itemOrig){
                            
                            $item = clone $itemOrig;
                            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento($item->getNreventom(), $this->_nrOrgTrabalho, 0, $this->_mesCompetencia);
                            if(in_array($evento->getNreventom(), $incideEventosNaoRecalcula)){
                                continue;   
                            }
                            
                            if(array_key_exists($evento->getNreventom(), $eventosInseridos)){
                                $evento = $this->_eventom->retornaEvento($evento->getNreventom());
                                $indice = $_itens_calculo_principal->procuraEvento($evento);
                                $itemOld = $_itens_calculo_principal->getItem($indice);
                                $ref = $itemOld->getVrreffolha() + $item->getVrreffolha();
                                $val = $itemOld->getVreventofolha() + $item->getVreventofolha();
                                $itemOld->setVrreffolha($ref);
                                $itemOld->setVreventofolha($val);
                            }else{
                                $item->setNritemcalcfolha(\HCM\Util\NovoCodigo::geraCodigo($this->_nrOrgTrabalho, 'FPA_ITECALCFOLHA', 1, 12, true));
                                $item->setIdrefevalorcalc('NORMAL');
                                $item->setNrcalculofolha($_itens_calculo_principal->getNrcalculofolha());
                                $item->setDtocorrencia(DateUtil::getDataDeString($dtCompetencia));
                                $_itens_calculo_principal->addItem($item);
                            }    
                                
                            $eventosInseridos[$evento->getNreventom()] = true;       
                            
                        }
                        
                        //$_itens_calculo_principal->persistList();
                        $this->_dependencia->analisaDependencia($vinculo);
                        
                        try {
                            $calculo->setListaProcessos($listaProcessos);
                            $result = $calculo->calculaFolhaComplementar($vinculo, $tp_calc_mov, $contador, $itemsOrig, $_itens_calculo_principal, $this->_mesCompetencia);
                            $resultadoCalcular[$vinculo->getNrvinculom()][] = $result;
                        } catch (\Exception $e) {
                            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
                            if (!$this->finalizaMensagens()) {
                                continue;
                            }
                            continue;
                        }
                    } else {
                        try {
                            
                            //ITENS SALARIO NOVO
                            $tp_calc_mov->retornaItensCalculo($vinculo);
                            $obj_tp_calc_mov = current($tp_calc_mov->retornaPosicaoTpMov($tp_calc_mov->retornaTpMovPrincipal()));
                            $_itens_calculo_principal = $obj_tp_calc_mov->getInstancia();
                            $_itens_calculo_principal->setNrTipoCalculo(1);
                            $itemsPrincipal = $_itens_calculo_principal->getItems();
                            //LIMPA ITENS CENARIO
                            if(count($itemsPrincipal) > 0){
                                $_itens_calculo_principal->removeAllItens();
                            }
                            
                            //Busca folha atual para deletar o evento de diferenca
                            $calc_folhaAtual = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
                                $this->_nrOrgTrabalho, FpatipomovimentEntity::TP_MOV_MENSAL, 1, 1, $_mesCompetenciaAtual, $_mesCompetenciaAtual
                            )[0];
                            $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->entityManager, $this->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
                            $itemCalcCompetenciaAtual->retornaItensPorVinculo($vinculo->getNrorg(), $vinculo->getNrvinculom(), $calc_folhaAtual->getNrcalculofolha());
                            
                            $evento_difSalario = $this->_eventom->retornaEvento(FpatipoassoceveEntity::ASSOC_DIF_SALARIO_DISSIDIO);
                            $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difSalario);
                            $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                            if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                                $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                            }
                            
                            $evento_difSalarioMaternidade = $this->_eventom->retornaEvento(334);
                            $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difSalarioMaternidade);
                            $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                            if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                                $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                            }
                            
                            $evento_difProvNaoIncide = $this->_eventom->retornaEvento(205510);
                            $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difProvNaoIncide);
                            $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                            if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                                $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                            }
                            
                            $evento_difDesconto = $this->_eventom->retornaEvento(205580);
                            $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difDesconto);
                            $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                            if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                                $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                            }
                            
                            $evento_difDescontoNaoIncide = $this->_eventom->retornaEvento(205559);
                            $indice = $itemCalcCompetenciaAtual->procuraEvento($evento_difDescontoNaoIncide);
                            $eveDifSalario = $itemCalcCompetenciaAtual->getItem($indice);
                            if($eveDifSalario && $eveDifSalario->getIdrefevalorcalc() == 'VALOR'){
                                $itemCalcCompetenciaAtual->excluiItem($eveDifSalario, $indice);
                            }
                        } catch (\Exception $e){
                            throw new \Exception($e->getMessage());
                        }
                    }
                    if (!$this->finalizaMensagens()) {
                        continue;
                    }
                    
                    $contador++;
                }
            } 
            if ($this->_msg_idabort === false) {
                try {
                    $fpacalculofolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrcalculofolha' => $nrcalcprincipal, 'nrorg' => $this->_nrOrgTrabalho));
                    if (($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO) or ($this->_msg_idstatus === MsgCalculo::ID_COM_ALERTA)) {
                        $fpacalculofolha->setIdstatusproc('E');
                    } else {
                        $fpacalculofolha->setIdstatusproc('C');
                    }
                    $fpacalculofolha->persist();
                    \Setforms2\Shared\Registry::getEntityManager()->flush();
                } catch (\Exception $e) {
                    \Setforms2\Shared\Registry::getEntityManager()->rollback();
                    throw $e;
                }
            }
            
            $simulacao['SIMULACAO'] = [];
            foreach($resultadoCalcular as $results){
                foreach($results as $result){
                    $simulacao['SIMULACAO'] = array_merge($simulacao['SIMULACAO'], $result['SIMULACAO'][0]);
                }
            }
            
            return array(
                'DEMONSTRATIVO' => [$simulacao],
                'CALCULO' => $nrcalcprincipal,
                'MENSAGENS' => ($this->_msg_idabort === true ? $this->_msg_mensagens : null),
                'ERRO' => ($this->_msg_idstatus === MsgCalculo::ID_COM_ERRO),
                'ABORT' => $this->_msg_idabort
            );
        } else {
            return null;
        }
    }
    
    private function _buscaDadosFormularioCalculo($dataSet){
        $this->_dsOrdem = trim($dataSet['DSORDEM']);
        $this->_dsInicio = is_string($dataSet['DSINICIO']) ? trim($dataSet['DSINICIO']) : $dataSet['DSINICIO'];
        $this->_dsFim = $dataSet['DSFIM'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_dtInicio = DateUtil::getDataDeString(substr($dataSet['DTINI'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $this->_dtFim = DateUtil::getDataDeString(substr($dataSet['DTFIM'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        $dataSet['IDDIFFERIAS'] = 'S';
    }

    private function finalizaMensagens() {
        $msgcalculo = MsgCalculo::getInstance()->getDados();
        if ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ERRO;
        } elseif ($msgcalculo['IDSTATUS'] === MsgCalculo::ID_COM_ALERTA and $this->_msg_idstatus != MsgCalculo::ID_COM_ERRO) {
            $this->_msg_idstatus = MsgCalculo::ID_COM_ALERTA;
        }
        if ($msgcalculo['IDABORT'] === true) {
            $this->_msg_idabort = true;
            $this->_msg_mensagens = $msgcalculo['MENSAGENS'];
            return false;
        } else {
            MsgCalculo::getInstance()->salvaMensagens();
        }
        MsgCalculo::getInstance()->clear();
        return true;
    }
}