<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\HCMParameters;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\VinculoRepository;

class RateioLancamentoVinculoController extends Crud {

    protected $rateioLancamentoVinculoService;
    protected $dataSourceName = 'fpa_rateiovinc';
    protected $dataSourceName1 = 'fpa_itrateiovinc_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, Connection $connection, EntityManager $entityManager, \HCM\FPA\Services\RateioLancamentoVinculoService $rateioLancamentoVinculoService, VinculoRepository $vinculoRepository){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->rateioLancamentoVinculoService = $rateioLancamentoVinculoService;
        $this->vinculoRepository = $vinculoRepository;
        parent::__construct($dataSourceManager);
    }

    public function getRateioLancamentoVinculo(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getItemRateio(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setRateioVinc(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeRateio = $this->rateioLancamentoVinculoService->buscaRateioExistente($row['NRORG'], $row['DTINIVIGENCIA']);
                if( $row['__is_new'] == false ){
                    if( $existeRateio ){
                        if( $existeRateio[0]["NRRATEIOVINC"] == $row["NRRATEIOVINC"] ){
                            $existeRateio = null;
                        }
                    }
                }
                
                if( !$existeRateio ){
                    if (array_key_exists('NRRATEIOVINC', $row)) {
                        if ($row['NRRATEIOVINC'] === '') {
                            $row['NRRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOVINC', 1, 12, true); 
                        }
                    } else {
                        $row['NRRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOVINC', 1, 12, true); 
                    }
                    parent::save($request, $response);
                } else {
                    $response->addNotification(new Notification("Já existe rateio com essa data de vigência", Notification::TYPE_ALERT));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setItemRateio(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_itrateiovinc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $mesmaEstruturaVinculo = $this->rateioLancamentoVinculoService->buscaMesmaEstruturaVinculo($row['NRORG'], $row['NRRATEIOVINC'], $row['NRVINCULOM'], $row['NRESTRUTURAM']);
                if( $row['__is_new'] == false ){
                    if( $mesmaEstruturaVinculo ){
                        if( $mesmaEstruturaVinculo[0]["NRITRATEIOVINC"] == $row["NRITRATEIOVINC"] ){
                            $mesmaEstruturaVinculo = null;
                        }
                    }
                }
                if( !$mesmaEstruturaVinculo ){
                    if (array_key_exists('NRITRATEIOVINC', $row)) {
                        if ($row['NRITRATEIOVINC'] === '') {
                            $row['NRITRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITRATEIOVINC', 1, 12, true); 
                        }
                    } else {
                        $row['NRITRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITRATEIOVINC', 1, 12, true);
                    }
                    parent::save($request, $response);
                } else {
                    $response->addNotification(new Notification("Já existe item com esse vínculo e com essa estrutura.", Notification::TYPE_ALERT));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setItensCadastroAutomatico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $mensagens = [];
                $auditoria = $this->getAuditoria();
                
                if( count($row['NRVINCULOM_LIST']) == 0 ){
                    $vinculos = $this->vinculoRepository->getVinculosByEstruturas($row['NRORG'], $this->hcmParameters->getDtCompetencia(), $row['NRESTRUTLEGAL_LIST'], $row['NRESTRUTGEREN_LIST']);
                    if( empty($vinculos) ){
                        $response->addNotification(new Notification("Nenhum vínculo foi encontrado para a(s) estrutura(s) escolhida(s).", 'noneFound'));
                        return -1;
                    }
                } else {
                    $vinculos = $row['VINCULOS'];
                }
                
                foreach( $vinculos as $vinculo ){
                    foreach( $row['ESTRUTURAS_CAD'] as $estrutura ){
                        $mesmaEstruturaVinculo = $this->rateioLancamentoVinculoService->buscaMesmaEstruturaVinculo($row['NRORG'], $row['NRRATEIOVINC'], $vinculo['NRVINCULOM'], $estrutura["NRESTRUTGEREN"]);
                        if( empty($mesmaEstruturaVinculo) ){
                            $row['NRITRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITRATEIOVINC', 1, 12, true);
                            $this->rateioLancamentoVinculoService->insereItemRateio($row['NRORG'], $row['NRITRATEIOVINC'], $row['NRRATEIOVINC'], $estrutura['NRPESO'], $vinculo['NRVINCULOM'], $estrutura["NRESTRUTGEREN"], $auditoria);
                        } else {
                            $mensagens[] = array(
                                "DS" => "Já existe item com o vínculo ".$vinculo['NMVINCULOM']." e com a estrutura ".$estrutura['NRESTRUTGEREN']
                            );
                        }
                    }
                }
                
                if( empty($mensagens) ){
                    $response->addNotification(new Notification("Cadastro automático realizado com sucesso!", Notification::TYPE_SUCCESS));
                } else {
                    $response->addNotification(new Notification("Cadastro automático realizado com alertas!", Notification::TYPE_ALERT));
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('alertCadastroAutomatico', $mensagens));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteRateioVinc(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $nrorg = $request->getDataSet()->getRows()[0]['NRORG'];
            $nrrateiovinc = $request->getDataSet()->getRows()[0]['NRRATEIOVINC'];

            $existeItens = $this->rateioLancamentoVinculoService->existeItensRateio($nrorg, $nrrateiovinc);
            
            if( empty($existeItens) ){
                parent::delete($request, $response);
            } else {
                $response->addNotification(new Notification("Existem itens relacionados a este registro, favor excluí-los.", Notification::TYPE_ALERT));
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteItemRateio(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "fpa_itrateiovinc";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
    }
    
    public function getAuditoria(){
        $auditoria = array();
        $auditoria['DTINCLUSAO'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
        $auditoria['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
        return $auditoria;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 