<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB\Itau;

use HCM\Util\FPA;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\FPA\Entity\FpaFormcnabEntity;
use HCM\FPA\Entity\FpaCnabmagnetEntity;

class Cnab341 {
    const newLine = '<br>';
    
    private $headerResult;  //Resultados de query, e de dados fixos (formulárionão formatado
    private $headerLoteResult;
    private $bodyResult;
    private $trailerLoteResult;
    private $trailerResult;

    private $headerString      = array(); //String de saída para arquivo txt já formatada
    private $headerLoteString  = array();
    private $bodyString        = array();
    private $trailerLoteString = array();
    private $trailerString     = array();

    private $nrSequencial;  //Sequencial dos registros
    private $vrTotal;       //Soma total dos valores

    private $queryHeader;   //Query's
    private $queryHeaderLote;
    private $queryBody;

    private $nrForm;        //Dados do formulário
    private $nrOrg;
    private $nrEvento;
    private $competencia;
    private $tpMovimento;
    private $modalidCalc;
    private $ocorrenCalc;
    private $ordem; //Escopo selecionado. Ordenamento que o dsIni e dsFim deverão considerar
    private $dsIni; //Inicio e fim do escopo selecionado
    private $dsFim;
    private $tpVinculo;
    private $idUtilDtPgtoItem;
    private $idIgnoraOcorrencia;
    private $dtPagamento;
    private $nrEstrutResp;
    //private $nrEstrutura;
    private $idGeraMagnet;
    private $cdBanco;
    //private $idTeste;
    private $cdOperador;
    //private $idatualizacad;
    private $cdConvenio;

    //Arrays de configuração de layouts
    private $layoutHeader = array( 'CODBANCO' => array( 'tipo' => 'N', 'ini'  =>   1, 'fim'  =>   3 ),   /*CODBANCO 001 003 9(03) 341*/
                                   'CODLOTE'  => array( 'tipo' => 'N', 'ini'  =>   4, 'fim'  =>   7 ),   /*CODLOTE  004 007 9(04) 0000*/
                                   'TIPREGIS' => array( 'tipo' => 'N', 'ini'  =>   8, 'fim'  =>   8 ),   /*TIPREGIS 008 008 9(01) 0*/
                                   'BRANCOS1' => array( 'tipo' => 'A', 'ini'  =>   9, 'fim'  =>  14 ),   /*BRANCOS1 009 014 X(06) */
                                   'LAYOUTVE' => array( 'tipo' => 'N', 'ini'  =>  15, 'fim'  =>  17 ),   /*LAYOUTVE 015 017 9(03) 080*/
                                   'TIPOINCR' => array( 'tipo' => 'N', 'ini'  =>  18, 'fim'  =>  18 ),   /*TIPOINCR 018 018 9(01) 1 = CPF 2 = CNPJ*/
                                   'NROINSCR' => array( 'tipo' => 'N', 'ini'  =>  19, 'fim'  =>  32 ),   /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO: CNPJ EMPRESA DEBITADA*/
                                   'BRANCOS2' => array( 'tipo' => 'A', 'ini'  =>  33, 'fim'  =>  52 ),   /*BRANCOS2 033 052 X(20) BRANCOS*/
                                   'NROAGENC' => array( 'tipo' => 'N', 'ini'  =>  53, 'fim'  =>  57 ),   /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA: NÚMERO AGÊNCIA DEBITADA*/
                                   'BRANCOS3' => array( 'tipo' => 'A', 'ini'  =>  58, 'fim'  =>  58 ),   /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
                                   'NROCONTA' => array( 'tipo' => 'N', 'ini'  =>  59, 'fim'  =>  70 ),   /*NROCONTA 059 070 9(12) NOTA 1 CONTA: NÚMERO DE C/C DEBITADA*/
                                   'BRANCOS4' => array( 'tipo' => 'A', 'ini'  =>  71, 'fim'  =>  71 ),   /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
                                   'NRODAC'   => array( 'tipo' => 'N', 'ini'  =>  72, 'fim'  =>  72 ),   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
                                   'NOMEMPRE' => array( 'tipo' => 'A', 'ini'  =>  73, 'fim'  => 102 ),   /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA*/
                                   'NOMBANCO' => array( 'tipo' => 'A', 'ini'  => 103, 'fim'  => 132 ),   /*NOMBANCO 103 132 X(30) NOME DO BANCO = BANCO ITAU*/
                                   'BRANCOS5' => array( 'tipo' => 'A', 'ini'  => 133, 'fim'  => 142 ),   /*BRANCOS5 133 142 X(10) COMPLEMENTO DE REGISTRO*/
                                   'ARQCODIG' => array( 'tipo' => 'N', 'ini'  => 143, 'fim'  => 143 ),   /*ARQCODIG 143 143 9(01) ARQUIVO-CÓDIGO CÓDIGO REMESSA/RETORNO 1=REMESSA 2=RETORNO = 1*/
                                   'DATGERAC' => array( 'tipo' => 'N', 'ini'  => 144, 'fim'  => 151 ),   /*DATGERAC 144 151 9(08) DDMMAAAA DATA DE GERAÇÃO DO ARQUIVO*/
                                   'HORGERAC' => array( 'tipo' => 'N', 'ini'  => 152, 'fim'  => 157 ),   /*HORGERAC 152 157 9(06) HHMMSS HORA DE GERAÇÃO DO ARQUIVO*/
                                   'ZEROS1'   => array( 'tipo' => 'N', 'ini'  => 158, 'fim'  => 166 ),   /*ZEROS1   158 166 9(09) COMPLEMENTO DE REGISTRO*/
                                   'DENSIDAD' => array( 'tipo' => 'N', 'ini'  => 167, 'fim'  => 171 ),   /*DENSIDAD 167 171 9(05) UNIDADE DE DENSIDADE DE GRAVAÇÃO DO ARQUIVO NOTA 2 = 00000*/
                                   'BRANCOS6' => array( 'tipo' => 'A', 'ini'  => 172, 'fim'  => 240 ) ); /*BRANCOS6 172 240 X(69) COMPLEMENTO DE REGISTRO*/
    private $layoutHeaderLote = array(  'CODBANCO' => array( 'tipo' => 'N', 'ini'  =>   1, 'fim'  =>   3 ),   /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
                                        'CODLOTE'  => array( 'tipo' => 'N', 'ini'  =>   4, 'fim'  =>   7 ),   /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
                                        'TIPREGIS' => array( 'tipo' => 'N', 'ini'  =>   8, 'fim'  =>   8 ),   /*TIPREGIS 008 008 9(01) 1 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
                                        'TIPOPERA' => array( 'tipo' => 'A', 'ini'  =>   9, 'fim'  =>   9 ),   /*TIPOPERA 009 009 X(01) C=CRÉDITO TIPO DE OPERAÇÃO TIPO DA OPERAÇÃO*/
                                        'TIPPAGTO' => array( 'tipo' => 'N', 'ini'  =>  10, 'fim'  =>  11 ),   /*TIPPAGTO 010 011 9(02) NOTA 4 TIPO DE PAGAMENTO TIPO DE PAGTO*/
                                        'FORPAGTO' => array( 'tipo' => 'N', 'ini'  =>  12, 'fim'  =>  13 ),   /*FORPAGTO 012 013 9(02) NOTA 5 FORMA DE PAGAMENTO FORMA DE PAGAMENTO*/
                                        'LAYOUTVE' => array( 'tipo' => 'N', 'ini'  =>  14, 'fim'  =>  16 ),   /*LAYOUTVE 014 016 9(03) 040 LAYOUT DO LOTE N DA VERSÃO DO LAYOUT DO LOTE*/
                                        'BRANCOS1' => array( 'tipo' => 'A', 'ini'  =>  17, 'fim'  =>  17 ),   /*BRANCOS1 017 017 X(01) COMPLEMENTO DE REGISTRO*/
                                        'TIPOINCR' => array( 'tipo' => 'N', 'ini'  =>  18, 'fim'  =>  18 ),   /*TIPOINCR 018 018 9(01) EMPRESA – INSCRIÇÃO TIPO INSCRIÇÃO EMPRESA DEBITADA 1 = CPF 2 = CNPJ*/
                                        'NROINSCR' => array( 'tipo' => 'N', 'ini'  =>  19, 'fim'  =>  32 ),   /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO CNPJ EMPRESA DEBITADA*/
                                        'BRANCOS2' => array( 'tipo' => 'A', 'ini'  =>  33, 'fim'  =>  52 ),   /*BRANCOS2 033 052 X(20) COMPLEMENTO DE REGISTRO*/
                                        'NROAGENC' => array( 'tipo' => 'N', 'ini'  =>  53, 'fim'  =>  57 ),   /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA NÚMERO AGÊNCIA DEBITADA*/
                                        'BRANCOS3' => array( 'tipo' => 'A', 'ini'  =>  58, 'fim'  =>  58 ),   /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
                                        'NROCONTA' => array( 'tipo' => 'N', 'ini'  =>  59, 'fim'  =>  70 ),   /*NROCONTA 059 070 9(12) NOTA 1 CONTA NÚMERO DE C/C DEBITADA*/
                                        'BRANCOS4' => array( 'tipo' => 'A', 'ini'  =>  71, 'fim'  =>  71 ),   /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
                                        'NRODAC'   => array( 'tipo' => 'N', 'ini'  =>  72, 'fim'  =>  72 ),   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA*/
                                        'NOMEMPRE' => array( 'tipo' => 'A', 'ini'  =>  73, 'fim'  => 102 ),   /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA NOME DA EMPRESA DEBITADA*/
                                        'FINALOTE' => array( 'tipo' => 'A', 'ini'  => 103, 'fim'  => 132 ),   /*FINALOTE 103 132 X(30) NOTA 6 FINALIDADE DO LOTE FINALIDADE DOS PAGTOS DO LOTE*/
                                        'HISTORCC' => array( 'tipo' => 'A', 'ini'  => 133, 'fim'  => 142 ),   /*HISTORCC 133 142 X(10) NOTA 7 HISTÓRICO DE C/C COMPLEMENTO HISTÓRICO C/C DEBITADA*/
                                        'ENDEMPRE' => array( 'tipo' => 'A', 'ini'  => 143, 'fim'  => 172 ),   /*ENDEMPRE 143 172 X(30) ENDEREÇO DA EMPRESA NOME DA RUA, AV, PÇA, ETC...*/
                                        'NROEMPRE' => array( 'tipo' => 'N', 'ini'  => 173, 'fim'  => 177 ),   /*NROEMPRE 173 177 9(05) NÚMERO NÚMERO DO LOCAL*/
                                        'COMEMPRE' => array( 'tipo' => 'A', 'ini'  => 178, 'fim'  => 192 ),   /*COMEMPRE 178 192 X(15) COMPLEMENTO. CASA, APTO, SALA, ETC...*/
                                        'MUNEMPRE' => array( 'tipo' => 'A', 'ini'  => 193, 'fim'  => 212 ),   /*MUNEMPRE 193 212 X(20) CIDADE NOME DA CIDADE*/
                                        'CEPEMPRE' => array( 'tipo' => 'N', 'ini'  => 213, 'fim'  => 220 ),   /*CEPEMPRE 213 220 9(08) CEP*/
                                        'ESTEMPRE' => array( 'tipo' => 'A', 'ini'  => 221, 'fim'  => 222 ),   /*ESTEMPRE 221 222 X(02) SIGLA DO ESTADO*/
                                        'BRANCOS5' => array( 'tipo' => 'A', 'ini'  => 223, 'fim'  => 230 ),   /*BRANCOS5 223 230 X(08) COMPLEMENTO DE REGISTRO*/
                                        'OCORRENC' => array( 'tipo' => 'A', 'ini'  => 231, 'fim'  => 240 ));  /*OCORRENC 231 240 X(10) NOTA 8 OCORRÊNCIAS: CÓDIGO OCORRÊNCIAS P/RETORNO*/
    private $layoutBody = array(    'CODBANCO' => array( 'tipo' => 'N', 'ini'  =>   1, 'fim'  =>   3 ),    /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
                                    'CODLOTE'  => array( 'tipo' => 'N', 'ini'  =>   4, 'fim'  =>   7 ),    /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
                                    'TIPREGIS' => array( 'tipo' => 'N', 'ini'  =>   8, 'fim'  =>   8 ),    /*TIPREGIS 008 008 9(01) 3 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
                                    'NROREGIS' => array( 'tipo' => 'N', 'ini'  =>   9, 'fim'  =>  13 ),    /*NROREGIS 009 013 9(05) NOTA 9 NÚMERO DO REGISTRO Nº SEQUENCIAL REGISTRO NO LOTE*/
                                    'CODSEGME' => array( 'tipo' => 'A', 'ini'  =>  14, 'fim'  =>  14 ),    /*CODSEGME 014 014 X(01) A SEGMENTO CÓDIGO SEGMENTO REG. DETALHE*/
                                    'TIPMOVIM' => array( 'tipo' => 'N', 'ini'  =>  15, 'fim'  =>  17 ),    /*TIPMOVIM 015 017 9(03) NOTA 10 TIPO DE MOVIMENTO TIPO DE MOVIMENTO = 000*/
                                    'ZEROS1'   => array( 'tipo' => 'N', 'ini'  =>  18, 'fim'  =>  20 ),    /*ZEROS1   018 020 9(03) COMPLEMENTO DE REGISTRO*/
                                    'BANFAVOR' => array( 'tipo' => 'N', 'ini'  =>  21, 'fim'  =>  23 ),    /*BANFAVOR 021 023 9(03) BANCO FAVORECIDO CÓDIGO BANCO FAVORECIDO*/
                                    'AGENCONT' => array( 'tipo' => 'A', 'ini'  =>  24, 'fim'  =>  43 ),    /*AGENCONT 024 043 X(20) NOTA 11 AGÊNCIA CONTA AGÊNCIA CONTA FAVORECIDO*/
                                    'NOMFAVOR' => array( 'tipo' => 'A', 'ini'  =>  44, 'fim'  =>  73 ),    /*NOMFAVOR 044 073 X(30) NOME DO FAVORECIDO*/
                                    'SEUNUMER' => array( 'tipo' => 'A', 'ini'  =>  74, 'fim'  =>  93 ),    /*SEUNUMER 074 093 X(20) SEU NÚMERO: Nº DOCTO ATRIBUÍDO PELA EMPRESA*/
                                    'DATPAGTO' => array( 'tipo' => 'D'/*'ini'  =>  94, 'fim'  => 101*/),   /*DATPAGTO 094 101 9(08) DDMMAAAA DATA DE PAGTO: DATA PREVISTA PARA PAGTO*/
                                    'TIPMOEDA' => array( 'tipo' => 'A', 'ini'  => 102, 'fim'  => 104 ),    /*TIPMOEDA 102 104 X(03) REA OU 009 TIPO TIPO DA MOEDA*/
                                    'ZEROS2'   => array( 'tipo' => 'N', 'ini'  => 105, 'fim'  => 119 ),    /*ZEROS2   105 119 9(15) COMPLEMENTO DE REGISTRO*/
                                    'VALPAGTO' => array( 'tipo' => 'N', 'ini'  => 120, 'fim'  => 134 ),    /*VALPAGTO 120 134 9(13)V9(02) VALOR DO PAGTO VALOR PREVISTO DO PAGTO*/
                                    'NOSNUMER' => array( 'tipo' => 'A', 'ini'  => 135, 'fim'  => 149 ),    /*NOSNUMER 135 149 X(15) NOTA 12 NOSSO NÚMERO Nº DOCTO ATRIBUÍDO PELO BANCO*/
                                    'BRANCOS1' => array( 'tipo' => 'A', 'ini'  => 150, 'fim'  => 154 ),    /*BRANCOS1 150 154 X(05) 0 COMPLEMENTO DE REGISTRO*/
                                    'DATAEFET' => array( 'tipo' => 'N', 'ini'  => 155, 'fim'  => 162 ),    /*DATAEFET 155 162 9(08) 0 DDMMAAAA DATA EFETIVA: DATA REAL EFETIVAÇÃO DO PAGTO*/
                                    'VALOEFET' => array( 'tipo' => 'N', 'ini'  => 163, 'fim'  => 177 ),    /*VALOEFET 163 177 9(13)V9(02) VALOR REAL EFETIVAÇÃO DO PAGTO*/
                                    'FINALIDA' => array( 'tipo' => 'A', 'ini'  => 178, 'fim'  => 195 ),    /*FINALIDA 178 195 X(18) NOTA 13 ' ' FINALIDADE DETALHE: INFORMAÇÃO COMPLEMENTAR P/ HIST. DE C/C*/
                                    'BRANCOS2' => array( 'tipo' => 'A', 'ini'  => 196, 'fim'  => 197 ),    /*BRANCOS2 196 197 X(2)' ' COMPLEMENTO DE REGISTRO*/
                                    'NUMDOCUM' => array( 'tipo' => 'N', 'ini'  => 198, 'fim'  => 203 ),    /*NUMDOCUM 198 203 9(6) NOTA 14 ' ' Nº DO DOCUMENTO Nº DO DOC/TED/ OP/ CHEQUE NO RETORNO*/
                                    'NUMINSCR' => array( 'tipo' => 'N', 'ini'  => 204, 'fim'  => 217 ),    /*NUMINSCR 204 217 9(14) NOTA 15 0 Nº DE INSCRIÇÃO N DE INSCRIÇÃO DO FAVORECIDO (CPF/CNPJ)*/
                                    'FINALDOC' => array( 'tipo' => 'A', 'ini'  => 218, 'fim'  => 219 ),    /*FINALDOC 218 219 X(02) NOTA 30 ' ' FINALIDADE DOC E STATUS: FINALIDADE DO DOC E STATUS DO FUNCIONÁRIO NA EMPRESA*/
                                    'FINALTED' => array( 'tipo' => 'A', 'ini'  => 220, 'fim'  => 224 ),    /*FINALTED 220 224 X(05) NOTA 26 ' ' FINALIDADE TED FINALIDADE DA TED*/
                                    'BRANCOS3' => array( 'tipo' => 'A', 'ini'  => 225, 'fim'  => 229 ),    /*BRANCOS3 225 229 X(05) COMPLEMENTO DE REGISTRO*/
                                    'AVISOFAV' => array( 'tipo' => 'A', 'ini'  => 230, 'fim'  => 230 ),    /*AVISOFAV 230 230 X(01) NOTA 16 ' ' AVISO AO FAVORECIDO*/
                                    'OCORRENC' => array( 'tipo' => 'A', 'ini'  => 231, 'fim'  => 240 ) );  /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGO OCORRÊNCIAS NO RETORNO*/
    private $layoutTrailerLote = array (    'CODBANCO' => array( 'tipo' => 'N', 'ini'  =>   1, 'fim'  =>   3 ), /*CODBANCO 001 003 9(03) 341 CÓDIGO DO BANCO CÓDIGO BANCO NA COMPENSAÇÃO*/
                                            'CODLOTE'  => array( 'tipo' => 'N', 'ini'  =>   4, 'fim'  =>   7 ), /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE LOTE DE SERVIÇO*/
                                            'TIPREGIS' => array( 'tipo' => 'N', 'ini'  =>   8, 'fim'  =>   8 ), /*TIPREGIS 008 008 9(01) 5 TIPO DE REGISTRO REGISTRO TRAILER DE LOTE*/
                                            'BRANCOS1' => array( 'tipo' => 'A', 'ini'  =>   9, 'fim'  =>  17 ), /*BRANCOS1 009 017 X(09) ' ' COMPLEMENTO DE REGISTRO*/
                                            'TOTALREG' => array( 'tipo' => 'N', 'ini'  =>  18, 'fim'  =>  23 ), /*TOTALREG 018 023 9(06) NOTA 17 TOTAL QTDE REGISTROS QTDE REGISTROS DO LOTE*/
                                            'TOTALVAL' => array( 'tipo' => 'N', 'ini'  =>  24, 'fim'  =>  41 ), /*TOTALVAL 024 041 9(16)V9(2) NOTA 17 TOTAL VALOR PAGTOS SOMA VALOR DOS PGTOS DO LOTE*/
                                            'ZEROS1'   => array( 'tipo' => 'N', 'ini'  =>  42, 'fim'  =>  59 ), /*ZEROS1   042 059 9(18) ZEROS COMPLEMENTO DE REGISTRO*/
                                            'BRANCOS2' => array( 'tipo' => 'A', 'ini'  =>  60, 'fim'  => 230 ), /*BRANCOS2 060 230 X(171) ' ' COMPLEMENTO DE REGISTRO*/
                                            'OCORRENC' => array( 'tipo' => 'A', 'ini'  => 231, 'fim'  => 240 ) ); /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGOS OCORRÊNCIAS P/ RETORNO*/
    private $layoutTrailer = array( 'CODBANCO' => array( 'tipo' => 'N', 'ini' =>   1, 'fim' =>   3 ),   /*CODBANCO 001 003 9(03) 341 CÓDIGO DO BANCO CÓDIGO BANCO NA COMPENSAÇÃO*/
                                    'CODLOTE'  => array( 'tipo' => 'N', 'ini' =>   4, 'fim' =>   7 ),   /*CODLOTE  004 007 9(04) 9999 CÓDIGO DO LOTE LOTE DE SERVIÇO*/
                                    'TIPREGIS' => array( 'tipo' => 'N', 'ini' =>   8, 'fim' =>   8 ),   /*TIPREGIS 008 008 9(01) 9 TIPO DE REGISTRO REGISTRO TRAILER DE ARQUIVO*/
                                    'BRANCOS1' => array( 'tipo' => 'A', 'ini' =>   9, 'fim' =>  17 ),   /*BRANCOS1 009 017 X(09)' ' COMPLEMENTO DE REGISTRO*/
                                    'QDELOTES' => array( 'tipo' => 'N', 'ini' =>  18, 'fim' =>  23 ),   /*QDELOTES 018 023 9(06) NOTA 17 1 TOTAL QTDE DE LOTES QTDE LOTES DO ARQUIVO*/
                                    'QDEREGIS' => array( 'tipo' => 'N', 'ini' =>  24, 'fim' =>  29 ),   /*QDEREGIS 024 029 9(06) NOTA 17 TOTAL QTDE REGISTROS Q T D E R E G ISTROS DO ARQUIVO*/
                                    'BRANCOS2' => array( 'tipo' => 'A', 'ini' =>  30, 'fim' => 240 ) ); /*BRANCOS2 030 240 X(211) COMPLEMENTO DE REGISTRO*/

    /**
     * Construtor. Adquiri os dados do formulário para as consultas
     * @param $params
     */
    public function __construct($entityManager, $hcmParameters, $connection,  $params){
        
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->nrForm       = $params['NRFORMCNAB'];    //Carrega os dados do formulário e ambiente
        $this->nrOrg        = $params['NRORG'];
        $this->nrEvento     = $params['NREVENTOM'];
        $this->competencia  = substr($hcmParameters->getDtCompetencia(), 0, 10);
        $this->tpMovimento  = $params['NRTIPOMOVIMENT'];
        $this->modalidCalc  = FPA::getNrModalidadeCalculo();
        $this->ocorrenCalc  = FPA::getOcorrenciaCalculo();
        $this->ordem        = $params['DSORDEM'];
        $this->dsIni        = is_array($params['DSINICIO']) ? implode($params['DSINICIO'], ",") : $params['DSINICIO'];
        $this->dsFim        = is_array($params['DSFIM']) ? implode($params['DSFIM'], ",") : $params['DSFIM'];
        $this->tpVinculo    = is_array($params['DSTIPOVINCULO']) ? implode($params['DSTIPOVINCULO'], ",") : $params['DSTIPOVINCULO'];
        $this->tpInscr      = $params['IDTPINSCRESPONS'];
        $this->nrEstrutResp = $params['NRESTRUTRESPONS'];
        $this->idGeraMagnet = $params['IDGERAMEIOMAG'];
        $this->cdBanco      = $params['CDBANCO'];
        //$this->idTeste      = $params['IDTESTE'];
        $this->cdConvenio   = $params['CODCONVENIO'];
        $this->cdOperador   = FPA::getCdOperador();

        $this->dtPagamento  = $params['DTPAGAMENTO'];
        $this->idUtilDtPgtoItem = $params['IDUTILDTPGTOITEM'];
        $this->idIgnoraOcorrencia = $params['IDIGNORAOCORRENCIA'];

       //$this->idatualizacad = $params['IDATUALIZACAD'] == 'S' ? 7 : 1; //1 pagamento a empregados; 7 abertura de cadastro

        $this->nrSequencial = -1;    //Número sequencial, pula as 2 primeiras linhas para funcionar no detalhe
        $this->vrTotal      = 0;    //Valor total dos registros

        //Query do header (arquivo)
        $this->queryHeader = <<<SQL
/*Header do Arquivo*/
SELECT
  /*CODBANCO 001 003 9(03) 341*/
  /*CODLOTE  004 007 9(04) 0000*/
  /*TIPREGIS 008 008 9(01) 0*/
  /*BRANCOS1 009 014 X(06) */
  /*LAYOUTVE 015 017 9(03) 080*/
  DECODE(EHL.CDCNPJESTRUT, NULL,
    DECODE(EHL.CDCPFESTRUTURA, NULL, NULL, 1),
    2)                                         TIPOINCR, /*TIPOINCR 018 018 9(01) 1 = CPF 2 = CNPJ*/
  NVL(EHL.CDCNPJESTRUT, EHL.CDCNPJESTRUT)      NROINSCR,  /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO: CNPJ EMPRESA DEBITADA*/
  /*BRANCOS2 033 052 X(20) BRANCOS*/
  CC.CDAGENCIA                                 NROAGENC, /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA: NÚMERO AGÊNCIA DEBITADA*/
  /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
  SUBSTR(LPAD( REPLACE( TRANSLATE( CC.CDCONTCORR, TRANSLATE('0' || CC.CDCONTCORR, '0123456789', '@' ), '@' ), '@', '' ), 13, '0'), 1, 12) NROCONTA, /*NROCONTA 059 070 9(12) NOTA 1 CONTA: NÚMERO DE C/C DEBITADA*/
  /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
  SUBSTR(CC.CDCONTCORR, -1)                    NRODAC,   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
  EHL.NMRAZSOCESTRUT                           NOMEMPRE, /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA*/
  /*NOMBANCO 103 132 X(30) NOME DO BANCO = BANCO ITAU*/
  /*BRANCOS5 133 142 X(10) COMPLEMENTO DE REGISTRO*/
  /*ARQCODIG 143 143 9(01) ARQUIVO-CÓDIGO CÓDIGO REMESSA/RETORNO 1=REMESSA 2=RETORNO = 1*/
  TO_CHAR(SYSDATE, 'DDMMYYYY')                 DATGERAC, /*DATGERAC 144 151 9(08) DDMMAAAA DATA DE GERAÇÃO DO ARQUIVO*/
  TO_CHAR(SYSDATE, 'HH24MISS')                 HORGERAC /*HORGERAC 152 157 9(06) HHMMSS HORA DE GERAÇÃO DO ARQUIVO*/
  /*ZEROS1   158 166 9(09) COMPLEMENTO DE REGISTRO*/
  /*DENSIDAD 167 171 9(05) UNIDADE DE DENSIDADE DE GRAVAÇÃO DO ARQUIVO NOTA 2 = 00000*/
  /*BRANCOS6 172 240 X(69) COMPLEMENTO DE REGISTRO*/
FROM ESTRUTURAM EML
JOIN ESTRUTURAH EHL ON EHL.NRORG        = EML.NRORG
                   AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                   AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= '$this->competencia' /*:P_COMPETENCIA*/
                                                AND NRORG        = EML.NRORG
                                                AND NRESTRUTURAM = EML.NRESTRUTURAM )
/*conta corrente e banco da estrutura legal*/
INNER JOIN CONTCORRPARC CC ON CC.NRORG          = EML.NRORG
                         AND CC.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO
                         --AND CC.NRTPCONTCORR   = 2 /*1 Conta de Movimentação, 2 CNAB, 3 Conta Salário, 4 Pensão Judicial*/
                         AND CC.IDATIVO        = 'S'
                         AND CC.CDBANCO = '341'
						             AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                   FROM CONTCORRPARC
                                                   WHERE NRORG         = CC.NRORG
                                                     AND IDATIVO       = 'S'
                                                     AND CDBANCO       = '341'
                                                     AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
													 AND NRTPCONTCORR  = CC.NRTPCONTCORR )
WHERE EML.NRORG        = $this->nrOrg /*:P_NRORG --19*/
  AND EML.NRESTRUTURAM = $this->nrEstrutResp /*:P_ESTRUTURA --1000539*/
SQL;

        //Query do header Lote
        $this->queryHeaderLote = <<<SQL
/*Header do Lote*/
SELECT
  /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
  /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
  /*TIPREGIS 008 008 9(01) 1 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
  /*TIPOPERA 009 009 X(01) C=CRÉDITO TIPO DE OPERAÇÃO TIPO DA OPERAÇÃO*/
  /*TIPPAGTO 010 011 9(02) NOTA 4 TIPO DE PAGAMENTO TIPO DE PAGTO*/
  /*FORPAGTO 012 013 9(02) NOTA 5 FORMA DE PAGAMENTO FORMA DE PAGAMENTO*/
  /*LAYOUTVE 014 016 9(03) 040 LAYOUT DO LOTE N DA VERSÃO DO LAYOUT DO LOTE*/
  /*BRANCOS1 017 017 X(01) COMPLEMENTO DE REGISTRO*/
  DECODE(EHL.CDCNPJESTRUT, NULL,
    DECODE(EHL.CDCPFESTRUTURA, NULL, NULL, 1),
    2)                                         TIPOINCR, /*TIPOINCR 018 018 9(01) EMPRESA – INSCRIÇÃO TIPO INSCRIÇÃO EMPRESA DEBITADA 1 = CPF 2 = CNPJ*/
  NVL(EHL.CDCNPJESTRUT, EHL.CDCNPJESTRUT)      NROINSCR, /*NROINSCR 019 032 9(14) NOTA 1 INSCRIÇÃO NÚMERO CNPJ EMPRESA DEBITADA*/
  /*BRANCOS2 033 052 X(20) COMPLEMENTO DE REGISTRO*/
  CC.CDAGENCIA                                 NROAGENC, /*NROAGENC 053 057 9(05) NOTA 1 AGÊNCIA NÚMERO AGÊNCIA DEBITADA*/
  /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
  SUBSTR(LPAD( REPLACE( TRANSLATE( CC.CDCONTCORR, TRANSLATE('0' || CC.CDCONTCORR, '0123456789', '@' ), '@' ), '@', '' ), 13, '0'), 1, 12) NROCONTA, /*NROCONTA 059 070 9(12) NOTA 1 CONTA: NÚMERO DE C/C DEBITADA*/
  /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
  SUBSTR(CC.CDCONTCORR, -1)                    NRODAC,   /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
  EHL.NMRAZSOCESTRUT                           NOMEMPRE, /*NOMEMPRE 073 102 X(30) NOME DA EMPRESA NOME DA EMPRESA DEBITADA*/
  DECODE($this->nrEvento /*:P_EVENTO*/, 3050, '08',
    DECODE($this->tpMovimento /*:P_TIPOMOVIMENT*/,  1, '01',
      10, '01', 4, '04', 19, '05', 2, '07',
      '10')) FINALOTE, /*FINALOTE 103 132 X(30) NOTA 6 FINALIDADE DO LOTE FINALIDADE DOS PAGTOS DO LOTE*/
  /*HISTORCC 133 142 X(10) NOTA 7 HISTÓRICO DE C/C COMPLEMENTO HISTÓRICO C/C DEBITADA*/
  TRANSLATE(
    DECODE(EP.CDLOGRADOURO, NULL, EP.DSENDERECO, EP.CDLOGRADOURO || ' ' || EP.DSENDERECO),
    'ÁÇÉÍÓÚÀÈÌÒÙÂÊÎÔÛÃÕËÜáçéíóúàèìòùâêîôûãõëü',
    'ACEIOUAEIOUAEIOUAOEUaceiouaeiouaeiouaoeu') ENDEMPRE, /*ENDEMPRE 143 172 X(30) ENDEREÇO DA EMPRESA NOME DA RUA, AV, PÇA, ETC...*/
  EP.NRIMOVELENDERECO                          NROEMPRE, /*NROEMPRE 173 177 9(05) NÚMERO NÚMERO DO LOCAL*/
  EP.DSCOMPLEENDERECO                          COMEMPRE, /*COMEMPRE 178 192 X(15) COMPLEMENTO. CASA, APTO, SALA, ETC...*/
  MU.NMMUNICIPIO                               MUNEMPRE, /*MUNEMPRE 193 212 X(20) CIDADE NOME DA CIDADE*/
  EP.NRCEPENDERECO                             CEPEMPRE, /*CEPEMPRE 213 220 9(08) CEP*/
  EP.SGESTADO                                  ESTEMPRE /*ESTEMPRE 221 222 X(02) SIGLA DO ESTADO*/
  /*BRANCOS5 223 230 X(08) COMPLEMENTO DE REGISTRO*/
  /*OCORRENC 231 240 X(10) NOTA 8 OCORRÊNCIAS: CÓDIGO OCORRÊNCIAS P/RETORNO*/
FROM ESTRUTURAM EML
JOIN ESTRUTURAH EHL ON EHL.NRORG        = EML.NRORG
                   AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                   AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= '$this->competencia' /*:P_COMPETENCIA*/
                                                AND NRORG        = EML.NRORG
                                                AND NRESTRUTURAM = EML.NRESTRUTURAM )
/*conta corrente e banco da estrutura legal*/
INNER JOIN CONTCORRPARC CC ON CC.NRORG          = EML.NRORG
                         AND CC.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO
                         --AND CC.NRTPCONTCORR   = 2 /*1 Conta de Movimentação, 2 CNAB, 3 Conta Salário, 4 Pensão Judicial*/
                         AND CC.IDATIVO        = 'S'
                         AND CC.CDBANCO = '341'
						             AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                   FROM CONTCORRPARC
                                                   WHERE NRORG         = CC.NRORG
                                                     AND IDATIVO       = 'S'
                                                     AND CDBANCO       = '341'
                                                     AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
													 AND NRTPCONTCORR  = CC.NRTPCONTCORR )
/*Endereço da estrutura legal*/
LEFT JOIN ENDERECOPARC EP ON EP.NRORG          = EML.NRORG
                         AND EP.NRPARCNEGOCIO  = EML.NRPARCNEGOCIO /*alterar aqui*/
                         AND EP.CDTIPOENDERECO = 'PRINCIPAL'
LEFT JOIN MUNICIPIO MU ON ( MU.NRORG = EP.NRORG OR MU.NRORG = 0 OR MU.NRORG IS NULL )
                      AND MU.CDPAIS      = EP.CDPAIS
                      AND MU.SGESTADO    = EP.SGESTADO
                      AND MU.CDMUNICIPIO = EP.CDMUNICIPIO
/**/
WHERE EML.NRORG        = $this->nrOrg /*:P_NRORG --19*/
  AND EML.NRESTRUTURAM = $this->nrEstrutResp /*:P_ESTRUTURA --1000539*/
SQL;

        //Query da remessa (body)
        $queryBodyDtItem = $this->idUtilDtPgtoItem == 'N' ? ' ' : "AND ICF.DTOCORRENCIA = '".$this->dtPagamento."'"; //Filtra data de ocorrência do item de cálculo caso seja necessário
        if ($this->idIgnoraOcorrencia == 'S') {
            $queryBodyOcorrencia = '';
        } else {
            $queryBodyOcorrencia = "AND CF.NROCORRECAL    = $this->ocorrenCalc";
        }
        $this->queryBody = 
<<<SQL
SELECT
  /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
  /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
  /*TIPREGIS 008 008 9(01) 3 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
  /*NROREGIS 009 013 9(05) NOTA 9 NÚMERO DO REGISTRO Nº SEQUENCIAL REGISTRO NO LOTE*/
  /*CODSEGME 014 014 X(01) A SEGMENTO CÓDIGO SEGMENTO REG. DETALHE*/
  /*TIPMOVIM 015 017 9(03) NOTA 10 TIPO DE MOVIMENTO TIPO DE MOVIMENTO = 000*/
  /*ZEROS1   018 020 9(03) COMPLEMENTO DE REGISTRO*/
  CC.CDBANCO        BANFAVOR, /*BANFAVOR 021 023 9(03) BANCO FAVORECIDO CÓDIGO BANCO FAVORECIDO*/
  LPAD(CC.CDAGENCIA, 5, '0')
    || ' ' || SUBSTR(LPAD( REPLACE( TRANSLATE( CC.CDCONTCORR, TRANSLATE('0' || CC.CDCONTCORR, '0123456789', '@' ), '@' ), '@', '' ), 13, '0'), 1, 12)
    || ' ' || SUBSTR(CC.CDCONTCORR, -1)                     AGENCONT, /*AGENCONT 024 043 X(20) NOTA 11 AGÊNCIA CONTA AGÊNCIA CONTA FAVORECIDO*/
  VM.NMVINCULOM     NOMFAVOR, /*NOMFAVOR 044 073 X(30) NOME DO FAVORECIDO*/
  VM.CDMATRICULA    SEUNUMER, /*SEUNUMER 074 093 X(20) SEU NÚMERO: Nº DOCTO ATRIBUÍDO PELA EMPRESA*/
  /*DATPAGTO 094 101 9(08) DDMMAAAA DATA DE PAGTO: DATA PREVISTA PARA PAGTO*/
  /*TIPMOEDA 102 104 X(03) REA OU 009 TIPO TIPO DA MOEDA*/
  /*ZEROS2   105 119 9(15) COMPLEMENTO DE REGISTRO*/
  ROUND(ICF.VREVENTOFOLHA * 100)                            VALPAGTO, /*VALPAGTO 120 134 9(13)V9(02) VALOR DO PAGTO VALOR PREVISTO DO PAGTO*/
  /*NOSNUMER 135 149 X(15) NOTA 12 NOSSO NÚMERO Nº DOCTO ATRIBUÍDO PELO BANCO*/
  /*BRANCOS1 150 154 X(05) 0 COMPLEMENTO DE REGISTRO*/
  /*DATAEFET 155 162 9(08) 0 DDMMAAAA DATA EFETIVA: DATA REAL EFETIVAÇÃO DO PAGTO*/
  /*VALOEFET 163 177 9(13)V9(02) 0 VALOR EFETIVO VALOR REAL EFETIVAÇÃO DO PAGTO*/
  /*FINALIDA 178 195 X(18) NOTA 13 ' ' FINALIDADE DETALHE: INFORMAÇÃO COMPLEMENTAR P/ HIST. DE C/C*/
  /*BRANCOS2 196 197 X(2)' ' COMPLEMENTO DE REGISTRO*/
  /*NUMDOCUM 198 203 9(6) NOTA 14 ' ' Nº DO DOCUMENTO Nº DO DOC/TED/ OP/ CHEQUE NO RETORNO*/
  PH.NRCPFPESSOA                                            NUMINSCR, /*NUMINSCR 204 217 9(14) NOTA 15 0 Nº DE INSCRIÇÃO N DE INSCRIÇÃO DO FAVORECIDO (CPF/CNPJ)*/
  /*FINALDOC 218 219 X(02) NOTA 30 ' ' FINALIDADE DOC E STATUS: FINALIDADE DO DOC E STATUS DO FUNCIONÁRIO NA EMPRESA*/
  /*FINALTED 220 224 X(05) NOTA 26 ' ' FINALIDADE TED FINALIDADE DA TED*/
  /*BRANCOS3 225 229 X(05) COMPLEMENTO DE REGISTRO*/
  /*AVISOFAV 230 230 X(01) NOTA 16 ' ' AVISO AO FAVORECIDO*/
  /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGO OCORRÊNCIAS NO RETORNO*/
  VM.NMVINCULOM NOME, VM.NRVINCULOM VINCULO /*Para os erros encontrarem*/
FROM GPE_VINCULOM VM
JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                    AND VH.NRVINCULOM = VM.NRVINCULOM
                    AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                              WHERE DTMESCOMPETENC <= '$this->competencia'
                                                AND NRORG      = VH.NRORG
                                                AND NRVINCULOM = VH.NRVINCULOM )
/*Estrutura Legal*/
JOIN ESTRUTURAM EML ON EML.NRORG        = VM.NRORG
                   AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
JOIN ESTRUTURAH EHL ON EHL.NRORG        = VM.NRORG
                   AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                   AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE DTMESCOMPETENC <= '$this->competencia'
                                                AND NRORG        = VM.NRORG
                                                AND NRESTRUTURAM = EML.NRESTRUTURAM )
/*Pessoa*/
JOIN GPE_PESSOA PE ON PE.NRORG    = VM.NRORG
                  AND PE.NRPESSOA = VM.NRPESSOA
JOIN GPE_PESSOAH PH ON PH.NRORG      = VM.NRORG
                    AND PH.NRPESSOA = VM.NRPESSOA
                    AND PH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                              WHERE DTMESCOMPETENC <= '$this->competencia'
                                                AND NRORG      = PH.NRORG
                                                AND NRPESSOA = PH.NRPESSOA )
INNER JOIN CONTCORRPARC CC ON CC.NRORG          = VM.NRORG
                         AND CC.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO
                         AND CC.CDBANCO = '341'
                         AND CC.IDATIVO        = 'S'
						             AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                   FROM CONTCORRPARC
                                                   WHERE NRORG         = CC.NRORG
                                                     AND IDATIVO       = 'S'
                                                     AND CDBANCO       = '341'
                                                     AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
													 AND NRTPCONTCORR  = CC.NRTPCONTCORR )
/*Itens de cálculo*/
JOIN FPA_CALCULOFOLHA CF ON CF.NRORG          = VM.NRORG
                        AND CF.NRTIPOMOVIMENT = $this->tpMovimento 
                        AND CF.NRTPMODALIDCAL = $this->modalidCalc
                        AND CF.DTOCORRENCIA   = '$this->competencia'
                        $queryBodyOcorrencia 
JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG          = CF.NRORG
                         AND ICF.NRVINCULOM     = VM.NRVINCULOM
                         AND ICF.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                         AND ICF.NREVENTOM      = $this->nrEvento 
                         AND ICF.VREVENTOFOLHA >= 0.005
                         $queryBodyDtItem
WHERE VM.NRORG         = $this->nrOrg 
SQL;

        $where = '';     //Query da remessa - Ajusta o complemento do where para filtrar e ordenar
        if($this->tpVinculo != ''){ 
            $where = "\r\n  AND INSTR("."'".$this->tpVinculo."'".", VM.NRTIPOVINCULOM||',' ) > 0";
        }
        
        switch( $this->ordem){
            case 'ALFABETICA':
                $where .= "\r\n  AND VM.NMVINCULOM BETWEEN '$this->dsIni' AND '$this->dsFim'\r\nORDER BY VM.NMVINCULOM";
                break;
            case 'MATRICULA':
                $where .= "\r\n  AND VM.CDMATRICULA BETWEEN '$this->dsIni' AND '$this->dsFim'\r\nORDER BY VM.CDMATRICULA";
                break;
            case 'DATA_ADMISSAO':
                $where .= "\r\n  AND VM.DTADMISSAOVINC BETWEEN '$this->dsIni' AND '$this->dsFim'\r\nORDER BY VM.DTADMISSAOVINC";
                break;
            case 'ESTRUTURA':
                if(isset($this->dsIni))
                    $where .= "\r\n  AND VH.NRESTRUTLEGAL = '$this->dsIni'";
                if(isset($this->dsFim))
                    $where .= "\r\n  AND VH.NRESTRUTGEREN = '$this->dsFim'";
                break;
            case 'LISTA_ESTRUTURA':
                if(isset($this->dsIni)){
                    $this->dsIni = is_array($this->dsIni) ? implode($this->dsIni, ",") : $this->dsIni;
                    $where .= "\r\n  AND VH.NRESTRUTLEGAL IN (".$this->dsIni.")";
                }
                if(isset($this->dsFim)){
                    $this->dsFim = is_array($this->dsFim) ? implode($this->dsFim, ",") : $this->dsFim;
                    $where .= "\r\n  AND VH.NRESTRUTGEREN IN (".$this->dsFim.")";
                }
                break;
            case 'LISTA_VINCULO':
                if($this->dsIni)
                $where .= "\r\n  AND VM.NRVINCULOM IN (" . $this->dsIni . ")";
                break;
            case 'LISTA_LOTE':
                if(isset($this->dsIni)){
                    $this->dsIni = is_array($this->dsIni) ? implode($this->dsIni, ",") : $this->dsIni;
                    $where .= "\r\n  AND VM.NRVINCULOM IN (SELECT NRVINCULOM FROM FPA_ITEMLOTEVINC WHERE NRORG = $this->nrOrg AND NRLOTEVINC IN($this->dsIni))";
                }
                break;
            case 'ESTRUTURA_SIND':
                if($this->dsIni)
                $where .= "\r\n  AND VH.NRESTRUTSIND IN (" . $this->dsIni . ")";
                break;
            case 'LISTA_NRESTRUTSIND':
                if($this->dsIni){
                    $this->dsIni = is_array($this->dsIni) ? implode(',',$this->dsIni) : $this->dsIni;
                    $where .= "\r\n  AND VH.NRESTRUTLEGAL IN (".$this->dsIni. ")";
                }
                if(isset($this->dsFim)){
                    $this->dsFim = is_array($this->dsFim) ? implode(',',$this->dsFim) : $this->dsFim;
                    $where .= "\r\n  AND VH.NRESTRUTSIND IN (".$this->dsFim. ")";
                }
                break;
            default:
                throw new \exception("Opção de ordenamento inválida ($this->ordem).");
        }
        $this->queryBody .= $where; //Completando a query com o restante do where e do order by
        $this->queryBody = "
            SELECT
                BANFAVOR, AGENCONT, NOMFAVOR, SEUNUMER, SUM(VALPAGTO) VALPAGTO, NUMINSCR, NOME, VINCULO
            FROM ($this->queryBody)
            GROUP BY
                BANFAVOR, AGENCONT, NOMFAVOR, SEUNUMER, NUMINSCR, NOME, VINCULO";
        
    }

    /**
     * Verifica se os campos obrigatórios estão preenchidos, caso contrário lança um erro
     * @throws \exception
     */
    private function verificaCamposRequeridos(){
        if($this->ordem == 'ALFABETICA' || $this->ordem == 'MATRICULA')
            $camposReq = array( 'Nº do evento'                      => $this->nrEvento,
                                'Tipo de movimento'                 => $this->tpMovimento,
                                'Ordem'                             => $this->ordem,
                                'Descrição inicial'                 => $this->dsIni,
                                'Descrição final'                   => $this->dsFim,
                                'Estrutura da Pessoa Responsável'   => $this->nrEstrutResp );
        else
            $camposReq = array( 'Nº do evento'                      => $this->nrEvento,
                                'Tipo de movimento'                 => $this->tpMovimento,
                                'Ordem'                             => $this->ordem,
                                'Descrição inicial'                 => $this->dsIni);
        $camposVazio = '';
        foreach($camposReq as $label => $campo ) //Procura campos requeridos não preenchidos
            if($campo == null)
                $camposVazio .= $label."\r\n";
        if($camposVazio != '')
            throw new \exception("Os seguintes campos devem estar preenchidos:\r\n".$camposVazio);
    }

    /**
     * Verifica se os dados obrigatórios de resposta da query estão todos preenchidos
     * @param array $resultQuery
     * @throws \exception
     */
    private function verificaResultQuery(array $resultQuery){
        $required = array(  'TIPOINCR' => 'Tipo de inscrição da empresa',
                            'NROINSCR' => 'Incrição (CNPJ/CPF) da empresa',
                            'NROAGENC' => 'Número da agência debitada',
                            'NROCONTA' => 'Número da conta debitada',
                            'NOMEMPRE' => 'Razão social da empresa',
                            'ENDEMPRE' => 'Endereço da empresa',
                            'NROEMPRE' => 'Número do endereço da empresa',
                            'MUNEMPRE' => 'Múnicípio (do endereço) da empresa',
                            'CEPEMPRE' => 'CEP da empresa',
                            //'BANFAVOR' => 'Número do banco favorecido',
                            'AGENCONT' => 'Agência do favorecido',
                            'NOMFAVOR' => 'Nome do favorecido (vínculo)',
                            'SEUNUMER' => 'Seu número (número de matrícula do vínculo)');
        $missing = '';
        $nrvinc = -1;
        $vn = false;            //Flag da quebra de linha do vínculo
        $vr = false;            //Contador/Flag do uso da vírgula
        foreach($resultQuery as $regKey => $regVal){    //Percorre todos os resultados da query
            foreach($regVal as $resKey => $resVal){     //Percorre todos os campos de um registro da query
                if(!isset($resVal) && isset($required[$resKey])){       //Procura resultados vazios e se encontar verifica se o mesmo é requerido
                    if($nrvinc != $resultQuery[$regKey]['VINCULO']){    //Verifica se os erros 'trocaram de vínculo'
                        $nrvinc   = $resultQuery[$regKey]['VINCULO'];   //Impede que a frase acima apareça repetidas vezes para o mesmo vínculo
                        $nmvinc   = $resultQuery[$regKey]['NOME'];      //Nome do vínculo
                        $vr       = false;                              //Reinicia o uso da vírgula
                        if($vn){
                            if(isset($nrvinc))                          //Verifica trata-se do header ou trailer onde não há registros de vínculos
                                $missing .= self::newLine."Vínculo $nrvinc $nmvinc: ";  //Identifica o vínculo em que ocorreu o(s) erro(s)
                            else
                                $missing .= self::newLine;              //Não exibi dados de vínculos, deve tratar-se de dados do header ou trailer
                        } else {
                            if(isset($nrvinc))                          //Verifica trata-se do header ou trailer onde não há registros de vínculos
                                $missing .= "Vínculo $nrvinc  $nmvinc: ";   //Primeira lihna que identifica o vínculo em que ocorreu o(s) erro(s)
                            $vn = true;                                 //Sinaliza que as demais linhas de identificação do vínculo são a 2ª linha em diante
                        }
                    }
                    if($vr){                                            //Verirfica se é o caso de se utilizar a ', '
                        $missing .= ', ';
                    } else {
                        $vr = true;                                     //Sinaliza o uso da vírgula para o próximo caso (vínculo)
                    }
                    $missing .= $required[$resKey];                     //Atualiza a mensagem de erro/campo faltante
                }
            }
        }
        if($missing != '')
            throw new \exception("Os seguintes campos devem estar preenchidos:".self::newLine.$missing);
    }
    /**
     * Adquiri os dados do header do arquivo
     * @return array
     * @throws \exception
     */
    private function getHeaderQueryResult(){
        try{
        
            return $this->connection->fetchAll($this->queryHeader);
        } catch(\Exception $e) {
            throw new \Exception('Erro na query, parte da remessa, a query usada foi esta: ' . $this->queryHeader);
        }
    }

    /**
     * Adquiri os dados fixos, do formulário e computados do cabeçalho do arquivo
     * @return array
     */
    private function getHeaderFixed(){
        return array(   'CODBANCO' => 341,          /*CODBANCO 001 003 9(03) 341*/
            'CODLOTE'  => 0,            /*CODLOTE  004 007 9(04) 0000*/
            'TIPREGIS' => 0,            /*TIPREGIS 008 008 9(01) 0*/
            'BRANCOS1' => ' ',          /*BRANCOS1 009 014 X(06) */
            'LAYOUTVE' => 80,           /*LAYOUTVE 015 017 9(03) 080*/
            'BRANCOS2' => ' ',          /*BRANCOS2 033 052 X(20) BRANCOS*/
            'BRANCOS3' => ' ',          /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
            'BRANCOS4' => ' ',          /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
            //'NRODAC'   => 5,            /*NRODAC   072 072 9(01) NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA = 5*/
            'NOMBANCO' => 'BANCO ITAU', /*NOMBANCO 103 132 X(30) NOME DO BANCO = BANCO ITAU*/
            'BRANCOS5' => ' ',          /*BRANCOS5 133 142 X(10) COMPLEMENTO DE REGISTRO*/
            'ARQCODIG' => 1,            /*ARQCODIG 143 143 9(01) ARQUIVO-CÓDIGO CÓDIGO REMESSA/RETORNO 1=REMESSA 2=RETORNO = 1*/
            'ZEROS1'   => 0,            /*ZEROS1   158 166 9(09) COMPLEMENTO DE REGISTRO*/
            'DENSIDAD' => 0,            /*DENSIDAD 167 171 9(05) UNIDADE DE DENSIDADE DE GRAVAÇÃO DO ARQUIVO NOTA 2 = 00000*/
            'BRANCOS6' => ' ');         /*BRANCOS6 172 240 X(69) COMPLEMENTO DE REGISTRO*/
    }

    /**
     * Adquiri os dados do header do lote
     * @return array
     * @throws \exception
     */
    private function getHeaderLoteQueryResult(){
        try{
            return $this->connection->fetchAll($this->queryHeaderLote);
        } catch(\Exception $e) {
            throw new \Exception('Erro na query, parte da remessa, a query usada foi esta: ' . $this->queryHeaderLote);
        }
    }


    /**
     * Adquiri os dados fixos, do formulário e computados do cabeçalho do lote
     * @return array
     */
    private function getHeaderLoteFixed(){
        return array(   'CODBANCO' => 341,          /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
                        'CODLOTE'  => 1,            /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
                        'TIPREGIS' => 1,            /*TIPREGIS 008 008 9(01) 1 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
                        'TIPOPERA' => 'C',          /*TIPOPERA 009 009 X(01) C=CRÉDITO TIPO DE OPERAÇÃO TIPO DA OPERAÇÃO*/
                        'TIPPAGTO' => 30,           /*TIPPAGTO 010 011 9(02) 30 NOTA 4 TIPO DE PAGAMENTO TIPO DE PAGTO*/
                        'FORPAGTO' => 1,            /*FORPAGTO 012 013 9(02) 01 NOTA 5 FORMA DE PAGAMENTO FORMA DE PAGAMENTO*/
                        'LAYOUTVE' => 40,           /*LAYOUTVE 014 016 9(03) 040 LAYOUT DO LOTE N DA VERSÃO DO LAYOUT DO LOTE*/
                        'BRANCOS1' => ' ',          /*BRANCOS1 017 017 X(01) COMPLEMENTO DE REGISTRO*/
                        'BRANCOS2' => ' ',          /*BRANCOS2 033 052 X(20) COMPLEMENTO DE REGISTRO*/
                        'BRANCOS3' => ' ',          /*BRANCOS3 058 058 X(01) COMPLEMENTO DE REGISTRO*/
                        'BRANCOS4' => ' ',          /*BRANCOS4 071 071 X(01) COMPLEMENTO DE REGISTRO*/
                        //'NRODAC'   => 5,            /*NRODAC   072 072 9(01) 5 NOTA 1 DAC DA AGÊNCIA/CONTA DEBITADA*/
                        'HISTORCC' => ' ',          /*HISTORCC 133 142 X(10) ' ' NOTA 7 HISTÓRICO DE C/C COMPLEMENTO HISTÓRICO C/C DEBITADA*/
                        'BRANCOS5' => ' ',          /*BRANCOS5 223 230 X(08) COMPLEMENTO DE REGISTRO*/
                        'OCORRENC' => ' ',          /*OCORRENC 231 240 X(10) NOTA 8 OCORRÊNCIAS: CÓDIGO OCORRÊNCIAS P/RETORNO*/
            );
    }

    /**
     * Adquiri os dados da remessa (bodyResult)
     * @return array
     * @throws \exception
     */
    private function getBodyQueryResult(){
        try{
            $result = $this->connection->fetchAll($this->queryBody);
            return $result;
        } catch(\Exception $e) {
            throw new \Exception('Erro na query, parte da remessa, a query usada foi esta: ' . $this->queryBody);
        }
    }

    /**
     * Adquiri os dados fixos, do formulário e computados da remessa (body) - Segmento A
     * @return array
     * @throws \Exception
     */
    private function getBodyFixed(){
        $dtPagamento = substr($this->dtPagamento, 0, 10);
        $dtPagamento = \HCM\Util\DateUtil::getDataDeString($dtPagamento);
        return array(   'CODBANCO' => 341,  /*CODBANCO 001 003 9(03) 341 CÓDIGO BANCO NA COMPENSAÇÃO*/
                        'CODLOTE'  => 1,    /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE: LOTE IDENTIFICAÇÃO DE PAGTOS = 0001*/
                        'TIPREGIS' => 3,    /*TIPREGIS 008 008 9(01) 3 TIPO DE REGISTRO REGISTRO HEADER DE LOTE*/
                                            /*NROREGIS 009 013 9(05) NOTA 9 NÚMERO DO REGISTRO Nº SEQUENCIAL REGISTRO NO LOTE*/
                        'CODSEGME' => 'A',  /*CODSEGME 014 014 X(01) A SEGMENTO CÓDIGO SEGMENTO REG. DETALHE*/
                        'TIPMOVIM' => 0,    /*TIPMOVIM 015 017 9(03) NOTA 10 TIPO DE MOVIMENTO TIPO DE MOVIMENTO = 000*/
                        'ZEROS1'   => 0,    /*ZEROS1   018 020 9(03) COMPLEMENTO DE REGISTRO*/
                        'DATPAGTO' => $dtPagamento, /*DATPAGTO 094 101 9(08) DDMMAAAA DATA DE PAGTO: DATA PREVISTA PARA PAGTO*/
                        'TIPMOEDA' => 'REA',        /*TIPMOEDA 102 104 X(03) REA OU 009 TIPO TIPO DA MOEDA*/
                        'ZEROS2'   => 0,    /*ZEROS2   105 119 9(15) COMPLEMENTO DE REGISTRO*/
                        'NOSNUMER' => ' ',  /*NOSNUMER 135 149 X(15) NOTA 12 NOSSO NÚMERO Nº DOCTO ATRIBUÍDO PELO BANCO*/
                        'BRANCOS1' => ' ',  /*BRANCOS1 150 154 X(05) ' ' COMPLEMENTO DE REGISTRO*/
                        'DATAEFET' => 0,    /*DATAEFET 155 162 9(08) 0 DDMMAAAA DATA EFETIVA: DATA REAL EFETIVAÇÃO DO PAGTO*/
                        'VALOEFET' => 0,    /*VALOEFET 163 177 9(13)V9(02) 0 VALOR EFETIVO VALOR REAL EFETIVAÇÃO DO PAGTO*/
                        'FINALIDA' => ' ',  /*FINALIDA 178 195 X(18) NOTA 13 ' ' FINALIDADE DETALHE: INFORMAÇÃO COMPLEMENTAR P/ HIST. DE C/C*/
                        'BRANCOS2' => ' ',  /*BRANCOS2 196 197 X(2)' ' COMPLEMENTO DE REGISTRO*/
                        'NUMDOCUM' => 0,    /*NUMDOCUM 198 203 9(6) NOTA 14 0 Nº DO DOCUMENTO Nº DO DOC/TED/ OP/ CHEQUE NO RETORNO*/
                        //'NUMINSCR' => 0,    /*NUMINSCR 204 217 9(14) NOTA 15 0 Nº DE INSCRIÇÃO N DE INSCRIÇÃO DO FAVORECIDO (CPF/CNPJ)*/
                        'FINALDOC' => ' ',  /*FINALDOC 218 219 X(02) NOTA 30 ' ' FINALIDADE DOC E STATUS: FINALIDADE DO DOC E STATUS DO FUNCIONÁRIO NA EMPRESA*/
                        'FINALTED' => ' ',  /*FINALTED 220 224 X(05) NOTA 26 ' ' FINALIDADE TED FINALIDADE DA TED*/
                        'BRANCOS3' => ' ',  /*BRANCOS3 225 229 X(05) COMPLEMENTO DE REGISTRO*/
                        'AVISOFAV' => ' ',  /*AVISOFAV 230 230 X(01) NOTA 16 ' ' AVISO AO FAVORECIDO*/
                        'OCORRENC' => ' '); /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGO OCORRÊNCIAS NO RETORNO*/

    }

    /**
     * @return array
     * Array da parte 'fixa' do layout do trailer do lote
     */
    private function getTrailerLoteFixed(){
        return array(   'CODBANCO' => 341,  /*CODBANCO 001 003 9(03) 341 CÓDIGO DO BANCO CÓDIGO BANCO NA COMPENSAÇÃO*/
                        'CODLOTE'  => 1,    /*CODLOTE  004 007 9(04) NOTA 3 CÓDIGO DO LOTE LOTE DE SERVIÇO*/
                        'TIPREGIS' => 5,    /*TIPREGIS 008 008 9(01) 5 TIPO DE REGISTRO REGISTRO TRAILER DE LOTE*/
                        'BRANCOS1' => ' ',  /*BRANCOS1 009 017 X(09) ' ' COMPLEMENTO DE REGISTRO*/
                        'TOTALREG' => $this->nrSequencial + 1,  /*TOTALREG 018 023 9(06) NOTA 17 TOTAL QTDE REGISTROS QTDE REGISTROS DO LOTE*/
                        'TOTALVAL' => $this->vrTotal,           /*TOTALVAL 024 041 9(16)V9(2) NOTA 17 TOTAL VALOR PAGTOS SOMA VALOR DOS PGTOS DO LOTE*/
                        'ZEROS1'   => 0,    /*ZEROS1   042 059 9(18) ZEROS COMPLEMENTO DE REGISTRO*/
                        'BRANCOS2' => ' ',  /*BRANCOS2 060 230 X(171) ' ' COMPLEMENTO DE REGISTRO*/
                        'OCORRENC' => ' '); /*OCORRENC 231 240 X(10) NOTA 8 ' ' OCORRÊNCIAS CÓDIGOS OCORRÊNCIAS P/ RETORNO*/
    }

    /**
     * @return array
     * Array da parte 'fixa' do layout do trailer do arquivo
     */
    private function getTrailerFixed(){
        return array(   'CODBANCO' => 341,  /*CODBANCO 001 003 9(03) 341 CÓDIGO DO BANCO CÓDIGO BANCO NA COMPENSAÇÃO*/
                        'CODLOTE'  => 9999, /*CODLOTE  004 007 9(04) 9999 CÓDIGO DO LOTE LOTE DE SERVIÇO*/
                        'TIPREGIS' => 9,    /*TIPREGIS 008 008 9(01) 9 TIPO DE REGISTRO REGISTRO TRAILER DE ARQUIVO*/
                        'BRANCOS1' => ' ',  /*BRANCOS1 009 017 X(09) ' ' COMPLEMENTO DE REGISTRO*/
                        'QDELOTES' => 1,    /*QDELOTES 018 023 9(06) NOTA 17 1 TOTAL QTDE DE LOTES QTDE LOTES DO ARQUIVO*/
                        'QDEREGIS' => $this->nrSequencial + 2,  /*QDEREGIS 024 029 9(06) NOTA 17 TOTAL QTDE REGISTROS QTDE REGISTROS DO ARQUIVO*/
                        'BRANCOS2' => ' ',  /*BRANCOS2 030 240 X(211) COMPLEMENTO DE REGISTRO*/
            );
    }

    /**
     * Mescla os dados fixos com o resultado da query em um novo array
     * @param array $dadosFixo
     * @param array $dadosResult
     * @return array
     */
    private function mesclaDadosFixoResult($dadosFixo, $dadosResult){
        foreach($dadosResult as $linha => $lixo) {
            foreach ($dadosFixo as $linha2 => $registros) {
                $dadosResult[$linha][$linha2] = $registros;
            }
            $dadosResult[$linha]['NROREGIS'] = $this->nrSequencial++; //Número sequencial do registro, todos os arrays (header, body e trailler) devem ter este campo: NRSEQUREG
        }
        return $dadosResult;
    }

    /**
     * @throws \Exception
     * Adquiri os dados para geração do magnético e do arquivo
     */
    public function gerar(){
        $this->verificaCamposRequeridos();              //Se houver algum campo requerido não preenchido este emitirá uma exceção
        $this->verificaMagnetico();                     //Retorna uma exceção caso já existam registros no magnético

        //Header do arquivo
        $Result = $this->getHeaderQueryResult();    //Adquiri o resultado via query do header
        $Fixo   = $this->getHeaderFixed();          //Adquiri os campos fixos do header

        $this->headerResult = $this->mesclaDadosFixoResult($Fixo, $Result);
        $this->verificaResultQuery($Result);        //Verifica/gerar execeção com relatórios de erros

        $campo = $this->headerResult[0];            //Obtendo a string de saída de acordo com o layout
        $this->headerString = Util::formataLayoutString($campo, $this->layoutHeader);

        //Header do lote
        $Result = $this->getHeaderLoteQueryResult();    //Adquiri o resultado via query do header
        $Fixo   = $this->getHeaderLoteFixed();          //Adquiri os campos fixos do header

        $this->headerLoteResult = $this->mesclaDadosFixoResult($Fixo, $Result);
        $this->verificaResultQuery($Result);        //Verifica/gerar execeção com relatórios de erros

        $campo = $this->headerLoteResult[0];
        $this->headerLoteString = Util::formataLayoutString($campo, $this->layoutHeaderLote);

        //Body - Registro Detalhe
        $Result = $this->getBodyQueryResult();      //Adquiri o resultado via query do body (remessa)
        $Fixo   = $this->getBodyFixed();            //Adquiri os campos fixos do body

        $this->bodyResult = $this->mesclaDadosFixoResult($Fixo, $Result);
        $this->verificaResultQuery($Result);        //Verifica/gerar execeção com relatórios de erros

        foreach( $this->bodyResult as $campo) {     //Obtendo a string de saÍda de acordo com o layout - BODY
            $this->vrTotal += (int)$campo['VALPAGTO'];
            $this->bodyString[] = Util::formataLayoutString($campo, $this->layoutBody);
        }

        //Trailer do Lote
        $Fixo = $this->getTrailerLoteFixed();
        $this->trailerLoteResult = $this->mesclaDadosFixoResult($Fixo, array(null));
        $campo = $this->trailerLoteResult[0];
        $this->trailerLoteString = Util::formataLayoutString($campo, $this->layoutTrailerLote);


        //Trailer do Arquivo
        $Fixo = $this->getTrailerFixed();
        $this->trailerResult = $this->mesclaDadosFixoResult($Fixo, array(null));
        $campo = $this->trailerResult[0];
        $this->trailerString = Util::formataLayoutString($campo, $this->layoutTrailer);

        //Persiste no magnético (se for o caso)
        $this->gerarMagnetico();
    }

    public function gerarArquivo($nomeArquivo, $extensao){
        
        //Se existirem dados da organização e operador este usará no nome do arquivo
        //if($this->nrOrg)  $nomeArquivo .= '-'.(string)$this->nrOrg;
        //if(FPA::getCdOperador()) $nomeArquivo .= '-'.(string)FPA::getCdOperador();
        $nomeArquivo .= '.'.(string)$extensao;
        FPA::escreveArquivo($nomeArquivo, array($this->headerString, $this->headerLoteString), true);
        FPA::escreveArquivo($nomeArquivo, $this->bodyString, true);
        FPA::escreveArquivo($nomeArquivo, array($this->trailerLoteString, $this->trailerString), true);
        return array(
            'ARQUIVO' => $nomeArquivo,
            'CALCULO' => null,
            'MENSAGENS' => null,
            'ERRO' => null,
            'ABORT' => null
        );
    }
    
    /**
     * @throws \Exception
     * Verifica se já existem registros no magnético (somente se a gravação do mesmo foi requerida). A verificação é
     * antecipada para evitar processamento (dos results e arrays de string) desnecessário.
     */
    public function verificaMagnetico(){
        if($this->idGeraMagnet == 'S') { //Verifica o magnético somente se a gravação do mesmo foi requerida
            $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
            $aux = $magnetico->retornaRegistros($this->competencia, $this->nrOrg, $this->cdBanco, $this->tpMovimento, $this->cdOperador);
            if ($aux)
                throw new \Exception("Já existem registros no magnético. O CNAB não foi gerado.");
        }
    }
    /**
     * @throws \Exception
     * @throws \TkValidationException
     * Grava dados no magnético somente se estes foram requeridos
     */
    public function gerarMagnetico(){
        if($this->idGeraMagnet == 'S'){ //Grava no magnético somente se o mesmo foi requerido
            $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
            for($i = 0, $n = count($this->bodyResult); $i < $n; ++$i) {
                $magnetico->insereRegistroLista($this->nrOrg, $this->cdOperador, $this->competencia, null, $this->tpInscr,
                    $this->headerResult[0]['NROINSCR'], $this->bodyString[$i], '341', $this->tpMovimento,
                    $this->bodyResult[$i]['NROREGIS'], $this->bodyResult[$i]['VALPAGTO'] / 100,
                    $this->bodyResult[$i]['VINCULO']);
                $magnetico->persist();
            }
        }
    }

    /**
     * Retrocede os registros do magnético
     */
    public function retroceder(){
        $magnetico = new FpaCnabmagnetEntity($this->entityManager, $this->hcmParameters);
        $magnetico->retroceder($this->nrOrg, $this->competencia, $this->cdBanco, $this->tpMovimento, $this->cdOperador);
    }

    /**
     * Retorna um array de array contendo todos os campos de resultado da query
     * @return mixed
     */
    public function getHeaderResult(){
        return $this->headerResult;
    }

    /**
     * Retorna um array de array contendo todos os campos de resultado da query
     * @return mixed
     */
    public function getBodyResult(){
        return $this->bodyResult;
    }

    /**
     * Retorna um array de array contendo todos os campos de resultado da query
     * @return mixed
     */
    public function getTraillerResult(){
        return $this->traillerResult;
    }

    /**
     * Retorna um array contendo a string do header (cabeçalho) formatada para geração do arquivo
     * @return array
     */
    public function getHeaderString(){
        return $this->headerString;
    }

    /**
     * Retorna um array contendo as strings do body (dados de tranferências) formatada para geração do arquivo
     * @return array
     */
    public function getBodyString(){
        return $this->bodyString;
    }

    /**
     * Retorna um array contendo a string do trailler (final do arquivo) formatada para geração do arquivo
     * @return array
     */
    public function getTraillerString(){
        return $this->traillerString;
    }
} 