<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S1005Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    public function getItensParaEnvioS1005($params) {
        try {
            $sql = <<<SQL
SELECT EH.NRESTRUTURAM as NRCHAVE,
       '' as NRTABELA,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       EH.DTMESCOMPETENC as DTMESCOMPETENC
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = EH.NRORG)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EH.NRESTRUTURAM AS NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-1005'
               JOIN ESTRUTURAH EH ON EH.NRORG = EC.NRORG
                                 AND EH.NRESTRUTURAM = EC.NRCHAVE
              WHERE EC.NRORG = :P_NRORG
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EH.NRESTRUTURAM
              UNION ALL
             SELECT EC.NRORG, EH.NRESTRUTURAM AS NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_CADASTRO EC
               JOIN ESTRUTURAH EH ON EH.NRORG = EC.NRORG
                                 AND EH.NRESTRUTURAM = EC.NRCHAVE
              WHERE EC.NRORG = :P_NRORG
                AND (EC.IDSTATUS = 'C' OR EC.IDSTATUS = 'A') /*QUE ESTIVER CONCLUIDO*/
                AND EC.IDATIVO = 'S' /*QUE ESTIVER ATIVO*/
                AND EC.IDOPERACAO <> 'E' /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRENVIOEXCL IS NULL /*QUE NAO TENHA EVENTO DE EXCLUSAO*/
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                AND EC.NMEVENTO = 'S-1005'
              GROUP BY EC.NRORG, EH.NRESTRUTURAM) ENV ON ENV.NRCHAVE = EH.NRESTRUTURAM
 WHERE EM.NRORG = :P_NRORG
   AND EM.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
   AND (EM.DTFIMVIGENCIA IS NULL OR EM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
   AND EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
   AND (EM.NRESTRUTURAM = :P_NRESTRUTURAM OR :P_NRESTRUTURAM IS NULL)
   AND SUBSTR(EH.CDCNPJESTRUT,1,8) = SUBSTR(:P_CDCNPJEMPREGADOR,1,8)
SQL;
            $params = array(
                "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
                "P_CDCNPJEMPREGADOR" => isset($params['CDCNPJEMPREGADOR']) ? $params['CDCNPJEMPREGADOR'] : null,
                "P_NRESTRUTURAM" => isset($params['NRESTRUTESTAB']) ? $params['NRESTRUTESTAB'] : null,
                "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            );
            return $this->connection->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosS1005($params){
        try{
            $dados = $this->getDadosS1005Principal($params['NRENVIO'], $params['NRCHAVE'], $params['NRTPAMBIENTE'],  $params['DTMESCOMPETENC']);
            return $dados;
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadosS1005Principal($nrenvio, $nrchave, $nrtpambiente, $dtmescompetenc) {
        $sql = <<<SQL
SELECT EM.NRESTRUTURAM CHAVE,
       'ID'||DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PN.CDTIPOINSCRICAO = 'CPF' OR EH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PN.NRINSCRICAOPARC ELSE SUBSTR(PN.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF' OR EH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PN.NRINSCRICAOPARC ELSE SUBSTR(PN.NRINSCRICAOPARC,1,8) END nrInsc,
       --infoEstab
       --ideEstab
       CASE
            WHEN (EH.CDCEIESTRUT IS NOT NULL AND EH.CDCAEPFESTRUT IS NULL) THEN '4'
            WHEN (EH.CDCEIESTRUT IS NULL AND EH.CDCAEPFESTRUT IS NOT NULL) THEN '3' 
            WHEN (EH.CDCEIESTRUT IS NOT NULL AND EH.CDCAEPFESTRUT IS NOT NULL) THEN '3'
            ELSE DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2')
       END ideEstab_tpInsc,
       NVL(NVL(EH.CDCAEPFESTRUT,EH.CDCEIESTRUT),PN.NRINSCRICAOPARC) ideEstab_nrInsc,
       TO_CHAR(GREATEST(EM.DTINIVIGENCIA,TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EM.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))),'YYYY-MM') iniValid,
       TO_CHAR(EM.DTFIMVIGENCIA, 'YYYY-MM') fimValid,
       --dadosEstab
       EH.CDCNAE cnaePrep,
       --aliqGilrat
       EH.VRPERCENTRAT aliqRat,
       CASE WHEN EH.IDENVFAPESO = 'S' THEN EH.VRFAP ELSE NULL END AS fap,
       --procAdmJudRat
       AJR.NRTPPROCESSADJU tpProc,
       AJR.CDPROCESSADMJUD nrProc,
       IPR.NRITEMPROCADMJUD codSusp,
       --procAdmJudFap
       AJF.NRTPPROCESSADJU procAdmJudFap_tpProc,
       AJF.CDPROCESSADMJUD procAdmJudFap_nrProc,
       IPF.NRITEMPROCADMJUD procAdmJudFap_codSusp,
       --infoCaepf
       '' infoCaepf,
       '' tpCaepf,
       '' infoObra,
       --indSubstPatrObra
       '' indSubstPatrObra,
       --infoTrab
       NVL(RP.CDESOCIAL, '0') regPt,
       --infoApr
       AJA.CDPROCESSADMJUD nrProcJud,
       --infoEntEduc getDadosInfoEntEduc
       --infoPCD
       AJP.CDPROCESSADMJUD infoPCD_nrProcJud,
       '' novaValidade,
       EM.NRESTRUTURAM
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                               FROM ESTRUTURAH H1
                                              WHERE H1.NRORG = EH.NRORG
                                                AND H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = EH.NRPARCNEGOCIO
                     AND PN.NRORG = EH.NRORG
  LEFT JOIN CNAEFISCAL CF ON CF.CDCNAE = EH.CDCNAE
  
  LEFT JOIN GPE_PROCESSADMJUD AJR ON EH.NRORG = AJR.NRORG AND EH.NRCDPROCESSORAT = AJR.NRPROCESSADMJUD
  LEFT JOIN GPE_PROCESSADMJUD AJF ON EH.NRORG = AJF.NRORG AND EH.NRCDPROCESSOFAP = AJF.NRPROCESSADMJUD
  LEFT JOIN GPE_PROCESSADMJUD AJA ON EH.NRORG = AJA.NRORG AND EH.NRCDCONTRATAPRENDIZ = AJA.NRPROCESSADMJUD
  LEFT JOIN GPE_PROCESSADMJUD AJP ON EH.NRORG = AJP.NRORG AND EH.NRCDPROCESSOCONTRATPCD = AJP.NRPROCESSADMJUD
  LEFT JOIN (SELECT NRPROCESSADMJUD, MIN(NRITEMPROCADMJUD) AS NRITEMPROCADMJUD
               FROM GPE_ITEMPROCADMJUD
              WHERE NRORG = :P_NRORG
              GROUP BY NRPROCESSADMJUD) IPR ON AJR.NRPROCESSADMJUD = IPR.NRPROCESSADMJUD
  LEFT JOIN (SELECT NRPROCESSADMJUD, MIN(NRITEMPROCADMJUD) AS NRITEMPROCADMJUD
               FROM GPE_ITEMPROCADMJUD
              WHERE NRORG = :P_NRORG
              GROUP BY NRPROCESSADMJUD) IPF ON AJF.NRPROCESSADMJUD = IPF.NRPROCESSADMJUD  
  LEFT JOIN (SELECT PA.NRORG, PA.NRESTRUTURAM, MAX(RP.CDESOCIAL) AS CDESOCIAL
               FROM GPE_PONTOACESSO PA
               JOIN GPE_TPREGIPONTO RP ON RP.NRTPREGIPONTO = PA.NRTPREGIPONTO
                                      AND RP.IDATIVO = 'S'
              WHERE PA.IDATIVO = 'S'
              GROUP BY PA.NRORG, PA.NRESTRUTURAM) RP ON RP.NRORG = EH.NRORG
                                                    AND RP.NRESTRUTURAM = EH.NRESTRUTURAM
  LEFT JOIN (SELECT HIERARQUIAORG.NRORG, HIERARQUIAORG.NRESTRUTURAMSUP, COUNT(DISTINCT ESTRUTHREL.CDCNPJESTRUT) QTDENTEDUC
               FROM HIERARQUIAORG
               JOIN ESTRUTURAM ESTRUTREL
                 ON HIERARQUIAORG.NRORG = ESTRUTREL.NRORG
                AND HIERARQUIAORG.NRESTRUTURAMREL = ESTRUTREL.NRESTRUTURAM
               JOIN ESTRUTURAH ESTRUTHREL
                 ON ESTRUTHREL.NRORG = ESTRUTREL.NRORG
                AND ESTRUTHREL.NRESTRUTURAM = ESTRUTREL.NRESTRUTURAM
                AND ESTRUTHREL.DTMESCOMPETENC = ( SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                   WHERE H1.NRORG = ESTRUTHREL.NRORG
                                                     AND H1.NRESTRUTURAM = ESTRUTHREL.NRESTRUTURAM
                                                     AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC )
              WHERE HIERARQUIAORG.NRTIPOHIERARQORG = 71
                AND HIERARQUIAORG.DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
                AND (HIERARQUIAORG.DTFIMVIGENCIA IS NULL OR HIERARQUIAORG.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
              GROUP BY HIERARQUIAORG.NRORG, HIERARQUIAORG.NRESTRUTURAMSUP) EE ON EE.NRORG = EH.NRORG
                                                                             AND EE.NRESTRUTURAMSUP = EH.NRESTRUTURAM
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRESTRUTURAM = :P_NRCHAVE
   AND EM.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidCadIniProdRest(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidCadIniProducao(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidCadIniNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave, 
            //"P_DTMESCOMPETENCIA" => $dtmescompetenc,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS1005InfoEntEduc($params){
        try{
            $dados = $this->getDadosInfoEntEduc($params['NRESTRUTURAM']);
            return $dados;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function getDadosInfoEntEduc($nrestruturam) {
        $sql = <<<SQL
SELECT DISTINCT ESTRUTHREL.CDCNPJESTRUT nrInsc
  FROM HIERARQUIAORG
  JOIN ESTRUTURAM ESTRUTREL
    ON HIERARQUIAORG.NRORG = ESTRUTREL.NRORG
   AND HIERARQUIAORG.NRESTRUTURAMREL = ESTRUTREL.NRESTRUTURAM
  JOIN ESTRUTURAH ESTRUTHREL
    ON ESTRUTHREL.NRORG = ESTRUTREL.NRORG
   AND ESTRUTHREL.NRESTRUTURAM = ESTRUTREL.NRESTRUTURAM
   AND ESTRUTHREL.DTMESCOMPETENC = ( SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                      WHERE H1.NRORG = ESTRUTHREL.NRORG
                                        AND H1.NRESTRUTURAM = ESTRUTHREL.NRESTRUTURAM
                                        AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC )
 WHERE HIERARQUIAORG.NRORG = :P_NRORG
   AND HIERARQUIAORG.NRESTRUTURAMSUP = :P_NRESTRUTURAM
   AND HIERARQUIAORG.NRTIPOHIERARQORG = 71
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => $nrestruturam,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
}