<?php

namespace HCM\PTO\Logic\AFD\Portaria671;

/**
 * Portaria 671
 * Gerador de nome para o arquivo AFD
 * 
 */

use HCM\PTO\Logic\AFD\GeradorNomeArquivo as GeradorNomeArquivoAFD;

class GeradorNomeArquivo extends GeradorNomeArquivoAFD {
    /**
     * Portaria 671
     * Gera o nome do arquivo AFD
     * @return string
     */
     public function gerarNome(): string {
        $dados = $this->buscaDados();
        return
            /* Ser nomeado pela junção da palavra "AFD" */ 'AFD' .
            /* Número de registro no INPI */ ($dados['INPI'] ?? '') .
            /* CNPJ/CPF do empregador */ ($dados['CPNJ'] ?? '') .
            /* "REP_P" */ 'REP_P' .
            '.txt';
     }
     
     /**
     * Portaria 671
     * Retorna informacoes importantes para a montagem do nome do arquivo
     * @return array
     */
     public function buscaDados(): array {
         $consulta = <<<SQL
            SELECT
                RPAD(NVL(CPNJ, '0'), 14, '0') CPNJ,
                RPAD(NVL(INPI, '0'), 17, '0') INPI
            FROM (
                SELECT
                  REPLACE(TRANSLATE( EH.CDCNPJESTRUT, TRANSLATE('0' || EH.CDCNPJESTRUT, '0123456789', '@' ), '@' ), '@', '' ) CPNJ,
                  REPLACE(TRANSLATE( VRH.VRREFERENCIA, TRANSLATE('0' || VRH.VRREFERENCIA, '0123456789', '@' ), '@' ), '@', '' ) INPI
                FROM ESTRUTURAM EM
                LEFT JOIN ESTRUTURAH EH ON EH.NRORG = EM.NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (
                      SELECT MAX(DTMESCOMPETENC)
                      FROM ESTRUTURAH
                      WHERE NRORG = EH.NRORG
                          AND NRESTRUTURAM = EH.NRESTRUTURAM
                          AND DTMESCOMPETENC <= :P_DTMESCOMPETENC
                    )
                LEFT JOIN FPA_VALORREFERENCIAM VRM ON VRM.NRORG IN ( :P_NRORG_PADRAO, EM.NRORG )
                    AND VRM.DSREFERENCIA = 'INPI'
                    AND VRM.DTINIVIGENCIA = ( SELECT MAX(DTINIVIGENCIA) FROM FPA_VALORREFERENCIAM
                                               WHERE NRORG          = VRM.NRORG
                                                 AND DSREFERENCIA   = VRM.DSREFERENCIA
                                                 AND DTINIVIGENCIA <= :P_DTMESCOMPETENC )
                LEFT JOIN FPA_VALORREFERENCIAH VRH ON VRH.NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                    AND VRH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_VALORREFERENCIAH
                                                WHERE NRVALORREFERENCIAM = VRM.NRVALORREFERENCIAM
                                                  AND DTMESCOMPETENC    <= :P_DTMESCOMPETENC )
                WHERE EM.NRORG = :P_NRORG
                    AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
            )
SQL;

        $params = [
            'P_NRORG' => $this->getParametros()->getNrorg(),
            'P_NRESTRUTURAM' => $this->getParametros()->getNrEstruturamEmpregador(),
            'P_DTMESCOMPETENC' => $this->getParametros()->getDtMesCompetenc(),
            'P_NRORG_PADRAO' => $this->getParametros()->getNrorgPadrao(),
        ];
        
        return $this->getEntityManager()->getConnection()->fetchAssoc(
            $consulta,
            $params
        );
     }
}