<?php
/**
 * Suprimentos\Logic\Exception
 *
 * @category Php
 * @package  Suprimentos\Logic\Exception
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Suprimentos\Logic\Exception;

/**
 * Class ExceptionCotacao
 *
 * @category Php
 * @package  Suprimentos\Logic\Exception
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class ExceptionCotacao extends \Geral\Logic\Exception
{
    private $_messageName;
    private $_params;
    private $_messageNameAcumulado;
    private $_paramsAcumulado;

    /**
     * Construtor
     *
     * @param string $messagemName Nome da Mensagem
     * @param array  $param        Parâmetro
     */
    public function __construct($messagemName, array $param = null)
    {
        parent::__construct($messagemName);
        $this->_messageName = $messagemName;
        $this->_params = $param;
    }

    /**
     * Retorna o nome da mensagem cadastrada no Setforms para esta excecao.
     *
     * @return string
     */
    public function getMessageName()
    {
        return $this->_messageName;
    }

    /**
     * Retorna o parametro da mensagem para esta excecao.
     *
     * @return string
     */
    public function getParam()
    {
        return $this->getParams();
    }

    /**
     * Acumula Mensagem
     * @param String $NmMensagem Nome da Mensagem
     * @param Array  $params     Parâmetros
     * @return null
     */
    public function acumulaMensagem($NmMensagem, $params)
    {
        $this->_messageNameAcumulado = $NmMensagem;
        $this->_paramsAcumulado = $params;
    }

    /**
     * Produto(s) informado(s) não cadastrado(s) para o comprador informado(s).
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroProdutoSemCadastro()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_ProdutoSemCadastro'));
    }

    /**
     * Fornecedor sem qualificação.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroFornecedorSemQualificacao()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_FornecedorSemQualificacao'));
    }

    /**
     * Produto(s) informado(s) não cadastrado(s) para o comprador e/ou fornecedor(es) informado(s)."
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroProdutoSemFornecedor()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_ProdutoSemFornecedor'));
    }

    /**
     * Marca sem qualificação.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroMarcaSemQualificacao()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_MarcaSemQualificacao'));
    }

    /**
     * Produto(s) informado(s) inativo(s).
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroProdutoInativo()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_ProdutoInativo'));
    }

    /**
     * Fornecedor(es) não atende(m) à(s) região(ões) informada(s).
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroRegiaoSemFornecedor()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_RegiaoSemFornecedor'));
    }

    /**
     * Preencha todos campos obrigatórios.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroCamposObrigatorios()
    {
        throw new self('msg_sup05200_campos_obrigatorios');
    }

    /**
     * É obrigatória pelo menos uma marca para o produto.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroMarcaNecessaria()
    {
        throw new self('msg_sup05200_marca_necessaria');
    }

    /**
     * Produto/Marca já cadastrados para a região e fornecedor posicionados.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroProdutoMarcaCadastrado()
    {
        throw new self('msg_sup05200_produto_marca_cadastrado');
    }

    /**
     * Fornecedor não qualificado ou com qualificação vencida.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroFornecedorNaoQualificado()
    {
        throw new self('msg_sup05200_fornecedor_nao_qualificado');
    }

    /**
     * Região já cadastrada.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroRegiaoCadastrada()
    {
        throw new self('msg_sup05200_regiao_cadastrada');
    }

    /**
     * Fornecedor já cadastrado.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroFornecedorCadastrado()
    {
        throw new self('msg_sup05200_fornecedor_cadastrado');
    }

    /**
     * Não existe solicitação.
     * @return null
     * @throws ExceptionCotacao
     */
    public static function erroNaoHaSolicitacao()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_nao_ha_solicitacao'));
    }

    /**
     * Operador não é um comprador
     * @return null
     * @throws ExceptionCotacao
     */
    public static function operadorNaoComprador()
    {
        throw new self(tk_get_message_by_name('msg_sup05200_operador_nao_comprador'));
    }

    /**
     * Cotação não pertence ao comprador
     * @return null
     * @throws ExceptionCotacao
     */
    public static function cotacaoNaoPertenceComprador(){
        throw new self(tk_get_message_by_name('msg_sup05200_cotacao_nao_pertence_comprador'));
    }

    /**
     * Operador sem autorização para alterar a cotação
     * @return null
     * @throws ExceptionCotacao
     */
    public static function operadorSemAutorizacao(){
        throw new self(tk_get_message_by_name('msg_sup05200_operador_sem_autorizacao'));
    }

    /**
     * Operador não relacionado ao comprador
     * @return null
     * @throws ExceptionCotacao
     */
    public static function operadorNaoRelacComprador(){
        throw new self(tk_get_message_by_name('msg_sup05200_operador_nao_relacionado_comprador'));
    }
}